@@ -7,7 +7,8 @@
 // A wrapper around DrawInfo, ImageInfo, and QuantizeInfo
 //
 
-#define MAGICK_IMPLEMENTATION  1
+#define MAGICKCORE_IMPLEMENTATION  1
+#define MAGICK_PLUSPLUS_IMPLEMENTATION 1
 
 #include "Magick++/Include.h"
 #include <string>
@@ -46,9 +47,9 @@ Magick::Options::Options( const Magick::Options& options_ )
 }
 
 // Construct using raw structures
-Magick::Options::Options( const MagickLib::ImageInfo* imageInfo_,
-                          const MagickLib::QuantizeInfo* quantizeInfo_,
-                          const MagickLib::DrawInfo* drawInfo_ )
+Magick::Options::Options( const MagickCore::ImageInfo* imageInfo_,
+                          const MagickCore::QuantizeInfo* quantizeInfo_,
+                          const MagickCore::DrawInfo* drawInfo_ )
 : _imageInfo(0),
   _quantizeInfo(0),
   _drawInfo(0)
@@ -144,6 +145,15 @@ Magick::Color Magick::Options::boxColor ( void ) const
   return Magick::Color( _drawInfo->undercolor );
 }
 
+void Magick::Options::colorspaceType ( Magick::ColorspaceType colorspace_ )
+{
+  _imageInfo->colorspace = colorspace_;
+}
+Magick::ColorspaceType Magick::Options::colorspaceType ( void ) const
+{
+  return static_cast<Magick::ColorspaceType>(_imageInfo->colorspace);
+}
+
 void Magick::Options::compressType ( CompressionType compressType_ )
 {
   _imageInfo->compression = compressType_;
@@ -232,35 +242,36 @@ std::string Magick::Options::fileName ( void ) const
 void Magick::Options::fillColor ( const Magick::Color &fillColor_ )
 {
   _drawInfo->fill = fillColor_;
+  if (fillColor_ == Magick::Color())
+    fillPattern((const MagickCore::Image*) NULL);
 }
 Magick::Color Magick::Options::fillColor ( void ) const
 {
   return _drawInfo->fill;
 }
-
 // Pattern image to use when filling objects
-void Magick::Options::fillPattern ( const MagickLib::Image *fillPattern_ )
+void Magick::Options::fillPattern ( const MagickCore::Image *fillPattern_ )
 {
   if ( _drawInfo->fill_pattern )
     {
       DestroyImageList( _drawInfo->fill_pattern );
       _drawInfo->fill_pattern = 0;
     }
-
   if ( fillPattern_ )
     {
       ExceptionInfo exceptionInfo;
       GetExceptionInfo( &exceptionInfo );
       _drawInfo->fill_pattern =
-	CloneImage( const_cast<MagickLib::Image*>(fillPattern_),
+	CloneImage( const_cast<MagickCore::Image*>(fillPattern_),
 		    0,
 		    0,
 		    static_cast<MagickBooleanType>(MagickTrue),
 		    &exceptionInfo );
       throwException( exceptionInfo );
+      (void) DestroyExceptionInfo( &exceptionInfo );
     }
 }
-const MagickLib::Image* Magick::Options::fillPattern ( void  ) const
+const MagickCore::Image* Magick::Options::fillPattern ( void  ) const
 {
   return _drawInfo->fill_pattern;
 }
@@ -314,6 +325,8 @@ std::string Magick::Options::format ( void ) const
   GetExceptionInfo(&exception);
   if ( *_imageInfo->magick != '\0' )
     magick_info = GetMagickInfo( _imageInfo->magick , &exception);
+  throwException( exception );
+  (void) DestroyExceptionInfo( &exception );
   
   if (( magick_info != 0 ) && 
       ( *magick_info->description != '\0' ))
@@ -335,13 +348,14 @@ void Magick::Options::magick ( const std::string &magick_ )
 {
   ExceptionInfo exception;
 
-  FormatString( _imageInfo->filename, "%.1024s:", magick_.c_str() );
+  FormatMagickString( _imageInfo->filename, MaxTextExtent, "%.1024s:", magick_.c_str() );
   GetExceptionInfo(&exception);
   SetImageInfo( _imageInfo, MagickTrue, &exception);
   if ( *_imageInfo->magick == '\0' )
     throwExceptionExplicit( OptionWarning,
 			    "Unrecognized image format",
 			    magick_.c_str() );
+  (void) DestroyExceptionInfo( &exception );
 }
 std::string Magick::Options::magick ( void ) const
 {
@@ -439,6 +453,21 @@ Magick::ResolutionType Magick::Options::resolutionUnits ( void ) const
   return static_cast<Magick::ResolutionType>(_imageInfo->units);
 }
 
+void Magick::Options::samplingFactor ( const std::string &samplingFactor_ )
+{
+  if ( samplingFactor_.length() == 0 )
+    _imageInfo->sampling_factor=(char *) RelinquishMagickMemory(_imageInfo->sampling_factor);
+  else
+    Magick::CloneString( &_imageInfo->sampling_factor, samplingFactor_ );
+}
+std::string Magick::Options::samplingFactor ( void ) const
+{
+  if ( _imageInfo->sampling_factor )
+    return std::string( _imageInfo->sampling_factor );
+
+  return std::string();
+}
+
 void Magick::Options::size ( const Geometry &geometry_ )
 {
   _imageInfo->size=(char *) RelinquishMagickMemory(_imageInfo->size);
@@ -482,7 +511,7 @@ void Magick::Options::strokeDashArray ( const double* strokeDashArray_ )
     {
       // Count elements in dash array
       unsigned int x;
-      for (x=0; strokeDashArray_[x]; x++);
+      for (x=0; strokeDashArray_[x]; x++) ;
       // Allocate elements
       _drawInfo->dash_pattern =
         static_cast<double*>(AcquireMagickMemory((x+1)*sizeof(double)));
@@ -539,7 +568,7 @@ unsigned int Magick::Options::strokeMiterLimit ( void ) const
 }
 
 // Pattern image to use for stroked outlines
-void Magick::Options::strokePattern ( const MagickLib::Image *strokePattern_ )
+void Magick::Options::strokePattern ( const MagickCore::Image *strokePattern_ )
 {
   if ( _drawInfo->stroke_pattern )
     {
@@ -552,15 +581,16 @@ void Magick::Options::strokePattern ( const MagickLib::Image *strokePattern_ )
       ExceptionInfo exceptionInfo;
       GetExceptionInfo( &exceptionInfo );
       _drawInfo->stroke_pattern =
-	CloneImage( const_cast<MagickLib::Image*>(strokePattern_),
+	CloneImage( const_cast<MagickCore::Image*>(strokePattern_),
 		    0,
 		    0,
 		    MagickTrue,
 		    &exceptionInfo );
       throwException( exceptionInfo );
+      (void) DestroyExceptionInfo( &exceptionInfo );
     }
 }
-const MagickLib::Image* Magick::Options::strokePattern ( void  ) const
+const MagickCore::Image* Magick::Options::strokePattern ( void  ) const
 {
   return _drawInfo->stroke_pattern;
 }
@@ -803,17 +833,17 @@ std::string Magick::Options::x11Display ( void ) const
 // Internal implementation methods.  Please do not use.
 //
 
-MagickLib::DrawInfo * Magick::Options::drawInfo( void )
+MagickCore::DrawInfo * Magick::Options::drawInfo( void )
 {
   return _drawInfo;
 }
 
-MagickLib::ImageInfo * Magick::Options::imageInfo( void )
+MagickCore::ImageInfo * Magick::Options::imageInfo( void )
 {
   return _imageInfo;
 }
 
-MagickLib::QuantizeInfo * Magick::Options::quantizeInfo( void )
+MagickCore::QuantizeInfo * Magick::Options::quantizeInfo( void )
 {
   return _quantizeInfo;
 }
