@@ -2,6 +2,7 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
 %                             AAA   RRRR   TTTTT                              %
 %                            A   A  R   R    T                                %
 %                            AAAAA  RRRR     T                                %
@@ -9,35 +10,26 @@
 %                            A   A  R  R     T                                %
 %                                                                             %
 %                                                                             %
-%                Read/Write PFS: 1st Publisher Image Format.                  %
+%                 Support PFS: 1st Publisher Clip Art Format                  %
 %                                                                             %
 %                              Software Design                                %
-%                              Jaroslav Fojtik                                %
-%                                January 2001                                 %
+%                                John Cristy                                  %
+%                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -50,67 +42,29 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
+#include "magick/color-private.h"
+#include "magick/colorspace.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/image.h"
 #include "magick/image-private.h"
 #include "magick/list.h"
-#include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
+
+/*
+  Forward declarations.
+*/
+static MagickBooleanType
+  WriteARTImage(const ImageInfo *,Image *);
 
-
-static void InsertRow(unsigned char *p,int y,Image *image)
-{
-unsigned long bit; long x;
-register PixelPacket *q;
-IndexPacket index;
-register IndexPacket *indexes;
-
-
- switch (image->depth)
-      {
-      case 1:  /* Convert bitmap scanline. */
-       {
-       q=SetImagePixels(image,0,y,image->columns,1);
-       if (q == (PixelPacket *) NULL)
-       break;
-       indexes=GetIndexes(image);
-       for (x=0; x < ((long) image->columns-7); x+=8)
-    {
-    for (bit=0; bit < 8; bit++)
-       {
-       index=(IndexPacket) (((*p) & (0x80 >> bit)) != 0 ? 0x01 : 0x00);
-       indexes[x+bit]=index;
-       *q++=image->colormap[index];
-       }
-    p++;
-    }
-       if ((image->columns % 8) != 0)
-     {
-     for (bit=0; bit < (image->columns % 8); bit++)
-         {
-         index=(IndexPacket) (((*p) & (0x80 >> bit)) != 0 ? 0x01 : 0x00);
-         indexes[x+bit]=index;
-         *q++=image->colormap[index];
-         }
-     p++;
-     }
-        if (SyncImagePixels(image) == MagickFalse)
-     break;
-/*            if (image->previous == (Image *) NULL)
-     if (QuantumTick(y,image->rows) != MagickFalse)
-       ProgressMonitor(LoadImageTag,image->rows-y-1,image->rows);*/
-      break;
-      }
-       }
-}
-
-
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -122,39 +76,51 @@ register IndexPacket *indexes;
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ReadARTImage() reads an ART X image file and returns it.  It
-%  allocates the memory necessary for the new Image structure and returns a
-%  pointer to the new image.
+%  ReadARTImage() reads an image of raw bits in LSB order and returns it.
+%  It allocates the memory necessary for the new Image structure and returns
+%  a pointer to the new image.
 %
 %  The format of the ReadARTImage method is:
 %
-%      Image *ReadARTImage(const ImageInfo *image_info,ExceptionInfo *exception)
+%      Image *ReadARTImage(const ImageInfo *image_info,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadARTImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  Image *image;
-  unsigned long width,height,dummy;
-  long ldblk;
-  unsigned char *BImgBuff=NULL;
-  unsigned char k;
+  Image
+    *image;
+
+  long
+    y;
+
+  QuantumInfo
+    *quantum_info;
+
+  QuantumType
+    quantum_type;
 
   MagickBooleanType
     status;
 
-  register long
-    i;
+  register PixelPacket
+    *q;
 
   ssize_t
     count;
 
+  size_t
+    length;
+
+  unsigned char
+    *pixels;
+
   /*
     Open image file.
   */
@@ -165,54 +131,60 @@ static Image *ReadARTImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  image->depth=1;
+  image->endian=MSBEndian;
+  (void) ReadBlobLSBShort(image);
+  image->columns=(unsigned long) ReadBlobLSBShort(image);
+  (void) ReadBlobLSBShort(image);
+  image->rows=(unsigned long) ReadBlobLSBShort(image);
   /*
-    Read ART image.
+    Initialize image colormap.
   */
-  dummy=ReadBlobLSBShort(image);
-  width=ReadBlobLSBShort(image);
-  dummy=ReadBlobLSBShort(image);
-  height=ReadBlobLSBShort(image);
-
-  ldblk=(long) ((width+7) / 8);
-  k=(unsigned char) ((-ldblk) & 0x01);
-  if (GetBlobSize(image) != (MagickSizeType) (8+(ldblk+(long) k)*height))
-    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
- image->columns=width;
- image->rows=height;
- image->depth=1;
- image->colors=1l << image->depth;
-
-/* printf("ART header checked OK %d,%d\n",image->colors,image->depth); */
-
- if (AllocateImageColormap(image,image->colors) == MagickFalse) goto NoMemory;
-
-/* ----- Load RLE compressed raster ----- */
- BImgBuff=(unsigned char *) AcquireMagickMemory((size_t) ldblk);  /*Ldblk was set in the check phase*/
- if (BImgBuff==NULL)
-NoMemory:
-  ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  for (i=0; i < (int) height; i++)
+  if (AcquireImageColormap(image,2) == MagickFalse)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  if (image_info->ping != MagickFalse)
+    {
+      (void) CloseBlob(image);
+      return(GetFirstImageInList(image));
+    }
+  /*
+    Convert bi-level image to pixel packets.
+  */
+  quantum_type=IndexQuantum;
+  quantum_info=AcquireQuantumInfo(image_info,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  pixels=GetQuantumPixels(quantum_info);
+  length=GetQuantumExtent(image,quantum_info,quantum_type);
+  for (y=0; y < (long) image->rows; y++)
   {
-    count=ReadBlob(image,(size_t) ldblk,BImgBuff);
-    if (count != (ssize_t) ldblk)
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
+    if (q == (PixelPacket *) NULL)
       break;
-    count=ReadBlob(image,(size_t) k,(unsigned char *) &dummy);
-    if (count != (ssize_t) k)
+    count=ReadBlob(image,length,pixels);
+    if (count != (ssize_t) length)
+      ThrowReaderException(CorruptImageError,"UnableToReadImageData");
+    (void) ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+      quantum_type,pixels,exception);
+    count=ReadBlob(image,(size_t) (-(ssize_t) length) & 0x01,pixels);
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
+      break;
+    if (SetImageProgress(image,LoadImageTag,y,image->rows) == MagickFalse)
       break;
-    InsertRow(BImgBuff,i,image);
   }
-  BImgBuff=(unsigned char *) RelinquishMagickMemory(BImgBuff);
-  if (i < (long) height)
+  SetQuantumImageType(image,quantum_type);
+  quantum_info=DestroyQuantumInfo(quantum_info);
+  if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -236,27 +208,23 @@ NoMemory:
 %
 %  The format of the RegisterARTImage method is:
 %
-%      RegisterARTImage(void)
+%      unsigned long RegisterARTImage(void)
 %
 */
-ModuleExport void RegisterARTImage(void)
+ModuleExport unsigned long RegisterARTImage(void)
 {
   MagickInfo
     *entry;
 
-  static const char
-    *ARTNote=
-    {
-      "Format originally used on the Macintosh (MacPaint?) and later used\n"
-      "for PFS: 1st Publisher clip art.  NOT the AOL ART format."
-    };
-
   entry=SetMagickInfo("ART");
-  entry->decoder=(DecoderHandler *) ReadARTImage;
-  entry->description=AcquireString("PFS: 1st Publisher");
-  entry->module=AcquireString("ART");
-  entry->note=AcquireString(ARTNote);
+  entry->decoder=(DecodeImageHandler *) ReadARTImage;
+  entry->encoder=(EncodeImageHandler *) WriteARTImage;
+  entry->raw=MagickTrue;
+  entry->adjoin=MagickFalse;
+  entry->description=ConstantString("PFS: 1st Publisher Clip Art");
+  entry->module=ConstantString("ART");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -282,3 +250,101 @@ ModuleExport void UnregisterARTImage(void)
 {
   (void) UnregisterMagickInfo("ART");
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   W r i t e A R T I m a g e                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  WriteARTImage() writes an image of raw bits in LSB order to a file.
+%
+%  The format of the WriteARTImage method is:
+%
+%      MagickBooleanType WriteARTImage(const ImageInfo *image_info,Image *image)
+%
+%  A description of each parameter follows.
+%
+%    o image_info: the image info.
+%
+%    o image:  The image.
+%
+*/
+static MagickBooleanType WriteARTImage(const ImageInfo *image_info,Image *image)
+{
+  long
+    y;
+
+  MagickBooleanType
+    status;
+
+  QuantumInfo
+    *quantum_info;
+
+  register const PixelPacket
+    *p;
+
+  ssize_t
+    count;
+
+  size_t
+    length;
+
+  unsigned char
+    *pixels;
+
+  /*
+    Open output image file.
+  */
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  if (status == MagickFalse)
+    return(status);
+  if ((image->columns > 65535UL) || (image->rows > 65535UL))
+    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
+  image->endian=MSBEndian;
+  image->depth=1;
+  (void) WriteBlobLSBShort(image,0);
+  (void) WriteBlobLSBShort(image,(unsigned short) image->columns);
+  (void) WriteBlobLSBShort(image,0);
+  (void) WriteBlobLSBShort(image,(unsigned short) image->rows);
+  if (image->colorspace != RGBColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
+  length=(image->columns+7)/8;
+  pixels=(unsigned char *) AcquireQuantumMemory(length,sizeof(*pixels));
+  if (pixels == (unsigned char *) NULL)
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  /*
+    Convert image to a bi-level image.
+  */
+  (void) SetImageType(image,BilevelType);
+  quantum_info=AcquireQuantumInfo(image_info,image);
+  for (y=0; y < (long) image->rows; y++)
+  {
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
+    if (p == (const PixelPacket *) NULL)
+      break;
+    (void) ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+      GrayQuantum,pixels,&image->exception);
+    count=WriteBlob(image,length,pixels);
+    if (count != (ssize_t) length)
+      ThrowWriterException(CorruptImageError,"UnableToWriteImageData");
+    count=WriteBlob(image,(size_t) (-(ssize_t) length) & 0x01,pixels);
+    if (SetImageProgress(image,SaveImageTag,y,image->rows) == MagickFalse)
+      break;
+  }
+  quantum_info=DestroyQuantumInfo(quantum_info);
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+  (void) CloseBlob(image);
+  return(MagickTrue);
+}
