@@ -10,14 +10,14 @@
 %                            DDDD   IIIII  BBBB                               %
 %                                                                             %
 %                                                                             %
-%                   Read/Write Windows DIB Image Format.                      %
+%                   Read/Write Windows DIB Image Format                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
@@ -56,8 +57,11 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 #include "magick/transform.h"
 
 /*
@@ -82,13 +86,11 @@ typedef struct _DIBInfo
     x_pixels,
     y_pixels,
     number_colors,
-    colors_important;
-
-  unsigned short
     red_mask,
     green_mask,
     blue_mask,
-    alpha_mask;
+    alpha_mask,
+    colors_important;
 
   long
     colorspace;
@@ -127,7 +129,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image: The address of a structure of type Image.
+%    o image: the address of a structure of type Image.
 %
 %    o compression:  A value of 1 means the compressed pixels are runlength
 %      encoded for a 256-color bitmap.  A value of 2 means a 16-color bitmap.
@@ -135,13 +137,23 @@ static MagickBooleanType
 %    o pixels:  The address of a byte (8 bits) array of pixel data created by
 %      the decoding process.
 %
-%
 */
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static MagickBooleanType DecodeImage(Image *image,
   const MagickBooleanType compression,unsigned char *pixels)
 {
 #if !defined(__WINDOWS__) || defined(__MINGW32__)
+#define BI_RGB  0
 #define BI_RLE8  1
+#define BI_RLE4  2
+#define BI_BITFIELDS  3
 #endif
 
   int
@@ -150,9 +162,6 @@ static MagickBooleanType DecodeImage(Image *image,
   long
     y;
 
-  MagickBooleanType
-    status;
-
   register long
     i,
     x;
@@ -184,7 +193,7 @@ static MagickBooleanType DecodeImage(Image *image,
       break;
     if (count != 0)
       {
-        count=Min((unsigned long) count,(size_t) (q-p));
+        count=(int) MagickMin((size_t) count,(size_t) (q-p));
         /*
           Encoded mode.
         */
@@ -237,7 +246,7 @@ static MagickBooleanType DecodeImage(Image *image,
             /*
               Absolute mode.
             */
-            count=Min((unsigned long) count,(size_t) (q-p));
+            count=(int) MagickMin((size_t) count,(size_t) (q-p));
             if (compression == BI_RLE8)
               for (i=0; i < count; i++)
                 *p++=(unsigned char) ReadBlobByte(image);
@@ -265,14 +274,8 @@ static MagickBooleanType DecodeImage(Image *image,
           }
         }
       }
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(LoadImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    if (SetImageProgress(image,LoadImageTag,y,image->rows) == MagickFalse)
+      break;
   }
   (void) ReadBlobByte(image);  /* end of line */
   (void) ReadBlobByte(image);
@@ -302,7 +305,7 @@ static MagickBooleanType DecodeImage(Image *image,
 %
 %    o image:  The image.
 %
-%    o bytes_per_line: The number of bytes in a scanline of compressed pixels
+%    o bytes_per_line: the number of bytes in a scanline of compressed pixels
 %
 %    o pixels:  The address of a byte (8 bits) array of pixel data created by
 %      the compression process.
@@ -310,7 +313,6 @@ static MagickBooleanType DecodeImage(Image *image,
 %    o compressed_pixels:  The address of a byte (8 bits) array of compressed
 %      pixel data.
 %
-%
 */
 static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
   const unsigned char *pixels,unsigned char *compressed_pixels)
@@ -318,9 +320,6 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
   long
     y;
 
-  MagickBooleanType
-    status;
-
   register const unsigned char
     *p;
 
@@ -362,14 +361,8 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
     */
     *q++=0x00;
     *q++=0x00;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(LoadImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
+    if (SetImageProgress(image,LoadImageTag,y,image->rows) == MagickFalse)
+      break;
   }
   /*
     End of bitmap.
@@ -404,7 +397,6 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsDIB(const unsigned char *magick,const size_t length)
 {
@@ -436,12 +428,26 @@ static MagickBooleanType IsDIB(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
+
+static inline long MagickAbsoluteValue(const long x)
+{
+  if (x < 0)
+    return(-x);
+  return(x);
+}
+
+static inline size_t MagickMax(const size_t x,const size_t y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
 static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   DIBInfo
@@ -497,7 +503,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -505,7 +511,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   /*
-    Determine if this is a DIB file.
+    Determine if this a DIB file.
   */
   (void) ResetMagickMemory(&dib_info,0,sizeof(dib_info));
   dib_info.size=ReadBlobLSBLong(image);
@@ -524,16 +530,16 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   dib_info.y_pixels=ReadBlobLSBLong(image);
   dib_info.number_colors=ReadBlobLSBLong(image);
   dib_info.colors_important=ReadBlobLSBLong(image);
-  if ((dib_info.compression == 3) && ((dib_info.bits_per_pixel == 16) ||
-      (dib_info.bits_per_pixel == 32)))
+  if ((dib_info.compression == BI_BITFIELDS) &&
+      ((dib_info.bits_per_pixel == 16) || (dib_info.bits_per_pixel == 32)))
     {
-      dib_info.red_mask=ReadBlobLSBShort(image);
-      dib_info.green_mask=ReadBlobLSBShort(image);
-      dib_info.blue_mask=ReadBlobLSBShort(image);
+      dib_info.red_mask=ReadBlobLSBLong(image);
+      dib_info.green_mask=ReadBlobLSBLong(image);
+      dib_info.blue_mask=ReadBlobLSBLong(image);
     }
   image->matte=dib_info.bits_per_pixel == 32 ? MagickTrue : MagickFalse;
-  image->columns=dib_info.width;
-  image->rows=AbsoluteValue(dib_info.height);
+  image->columns=(unsigned long) MagickAbsoluteValue(dib_info.width);
+  image->rows=(unsigned long) MagickAbsoluteValue(dib_info.height);
   image->depth=8;
   if ((dib_info.number_colors != 0) || (dib_info.bits_per_pixel < 16))
     {
@@ -561,6 +567,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (image->storage_class == PseudoClass)
     {
       size_t
+        length,
         packet_size;
 
       unsigned char
@@ -569,9 +576,11 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Read DIB raster colormap.
       */
-      if (AllocateImageColormap(image,image->colors) == MagickFalse)
+      if (AcquireImageColormap(image,image->colors) == MagickFalse)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      dib_colormap=(unsigned char *) AcquireMagickMemory(4*image->colors);
+      length=(size_t) image->colors;
+      dib_colormap=(unsigned char *) AcquireQuantumMemory(length,
+        4*sizeof(*dib_colormap));
       if (dib_colormap == (unsigned char *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       packet_size=4;
@@ -592,15 +601,16 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Read image data.
   */
-  if (dib_info.compression == 2)
+  if (dib_info.compression == BI_RLE4)
     dib_info.bits_per_pixel<<=1;
   bytes_per_line=4*((image->columns*dib_info.bits_per_pixel+31)/32);
   length=bytes_per_line*image->rows;
-  pixels=(unsigned char *) AcquireMagickMemory((size_t)
-    Max(bytes_per_line,image->columns+256)*image->rows*sizeof(*pixels));
+  pixels=(unsigned char *) AcquireQuantumMemory((size_t) image->rows,
+    MagickMax(bytes_per_line,image->columns+256UL)*sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  if ((dib_info.compression == 0) || (dib_info.compression == 3))
+  if ((dib_info.compression == BI_RGB) ||
+      (dib_info.compression == BI_BITFIELDS))
     {
       count=ReadBlob(image,length,pixels);
       if (count != (ssize_t) (length))
@@ -620,8 +630,8 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Initialize image structure.
   */
   image->units=PixelsPerCentimeterResolution;
-  image->x_resolution=dib_info.x_pixels/100.0;
-  image->y_resolution=dib_info.y_pixels/100.0;
+  image->x_resolution=(double) dib_info.x_pixels/100.0;
+  image->y_resolution=(double) dib_info.y_pixels/100.0;
   /*
     Convert DIB raster image to pixel packets.
   */
@@ -635,17 +645,17 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       for (y=(long) image->rows-1; y >= 0; y--)
       {
         p=pixels+(image->rows-y-1)*bytes_per_line;
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=GetAuthenticIndexQueue(image);
         for (x=0; x < ((long) image->columns-7); x+=8)
         {
           for (bit=0; bit < 8; bit++)
           {
-            index=((*p) & (0x80 >> bit) ? 0x01 : 0x00);
+            index=(IndexPacket) ((*p) & (0x80 >> bit) ? 0x01 : 0x00);
             indexes[x+bit]=index;
-            *q++=image->colormap[index];
+            *q++=image->colormap[(long) index];
           }
           p++;
         }
@@ -653,23 +663,21 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             for (bit=0; bit < (long) (image->columns % 8); bit++)
             {
-              index=((*p) & (0x80 >> bit) ? 0x01 : 0x00);
+              index=(IndexPacket) ((*p) & (0x80 >> bit) ? 0x01 : 0x00);
               indexes[x+bit]=index;
-              *q++=image->colormap[index];
+              *q++=image->colormap[(long) index];
             }
             p++;
           }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
-                image->rows,image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+          {
+            status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+              image->rows);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -681,38 +689,36 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       for (y=(long) image->rows-1; y >= 0; y--)
       {
         p=pixels+(image->rows-y-1)*bytes_per_line;
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=GetAuthenticIndexQueue(image);
         for (x=0; x < ((long) image->columns-1); x+=2)
         {
           index=ConstrainColormapIndex(image,(*p >> 4) & 0xf);
           indexes[x]=index;
-          *q++=image->colormap[index];
+          *q++=image->colormap[(long) index];
           index=ConstrainColormapIndex(image,*p & 0xf);
           indexes[x+1]=index;
-          *q++=image->colormap[index];
+          *q++=image->colormap[(long) index];
           p++;
         }
         if ((image->columns % 2) != 0)
           {
             index=ConstrainColormapIndex(image,(*p >> 4) & 0xf);
             indexes[x]=index;
-            *q++=image->colormap[index];
+            *q++=image->colormap[(long) index];
             p++;
           }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
-                image->rows,image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+          {
+            status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+              image->rows);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -721,34 +727,33 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Convert PseudoColor scanline.
       */
-      if ((dib_info.compression == 1) || (dib_info.compression == 2))
+      if ((dib_info.compression == BI_RLE8) ||
+          (dib_info.compression == BI_RLE4))
         bytes_per_line=image->columns;
       for (y=(long) image->rows-1; y >= 0; y--)
       {
         p=pixels+(image->rows-y-1)*bytes_per_line;
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=GetAuthenticIndexQueue(image);
         for (x=0; x < (long) image->columns; x++)
         {
           index=ConstrainColormapIndex(image,*p);
           indexes[x]=index;
-          *q=image->colormap[index];
+          *q=image->colormap[(long) index];
           p++;
           q++;
         }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
-                image->rows,image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+          {
+            status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+              image->rows);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -761,12 +766,12 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         Convert PseudoColor scanline.
       */
       image->storage_class=DirectClass;
-      if (dib_info.compression == 1)
+      if (dib_info.compression == BI_RLE8)
         bytes_per_line=2*image->columns;
       for (y=(long) image->rows-1; y >= 0; y--)
       {
         p=pixels+(image->rows-y-1)*bytes_per_line;
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
         for (x=0; x < (long) image->columns; x++)
@@ -775,29 +780,33 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
           word|=(*p++ << 8);
           if (dib_info.red_mask == 0)
             {
-              q->red=ScaleCharToQuantum(ScaleColor5to8((word >> 10) & 0x1f));
-              q->green=ScaleCharToQuantum(ScaleColor5to8((word >> 5) & 0x1f));
-              q->blue=ScaleCharToQuantum(ScaleColor5to8(word & 0x1f));
+              q->red=ScaleCharToQuantum(ScaleColor5to8((unsigned char)
+                ((word >> 10) & 0x1f)));
+              q->green=ScaleCharToQuantum(ScaleColor5to8((unsigned char)
+                ((word >> 5) & 0x1f)));
+              q->blue=ScaleCharToQuantum(ScaleColor5to8((unsigned char)
+                (word & 0x1f)));
             }
           else
             {
-              q->red=ScaleCharToQuantum(ScaleColor5to8((word >> 11) & 0x1f));
-              q->green=ScaleCharToQuantum(ScaleColor6to8((word >> 5) & 0x3f));
-              q->blue=ScaleCharToQuantum(ScaleColor5to8(word & 0x1f));
+              q->red=ScaleCharToQuantum(ScaleColor5to8((unsigned char)
+                ((word >> 11) & 0x1f)));
+              q->green=ScaleCharToQuantum(ScaleColor6to8((unsigned char)
+                ((word >> 5) & 0x3f)));
+              q->blue=ScaleCharToQuantum(ScaleColor5to8((unsigned char)
+                (word & 0x1f)));
             }
           q++;
         }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
-                image->rows,image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+          {
+            status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+              image->rows);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -810,7 +819,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       for (y=(long) image->rows-1; y >= 0; y--)
       {
         p=pixels+(image->rows-y-1)*bytes_per_line;
-        q=SetImagePixels(image,0,y,image->columns,1);
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
         for (x=0; x < (long) image->columns; x++)
@@ -822,17 +831,15 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q->opacity=ScaleCharToQuantum(*p++);
           q++;
         }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
         if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(LoadImageTag,image->rows-y-1,
-                image->rows,image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+          {
+            status=SetImageProgress(image,LoadImageTag,image->rows-y-1,
+              image->rows);
+            if (status == MagickFalse)
+              break;
+          }
       }
       break;
     }
@@ -860,7 +867,7 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImage(image);
       image=flipped_image;
     }
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -884,24 +891,25 @@ static Image *ReadDIBImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterDIBImage method is:
 %
-%      RegisterDIBImage(void)
+%      unsigned long RegisterDIBImage(void)
 %
 */
-ModuleExport void RegisterDIBImage(void)
+ModuleExport unsigned long RegisterDIBImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("DIB");
-  entry->decoder=(DecoderHandler *) ReadDIBImage;
-  entry->encoder=(EncoderHandler *) WriteDIBImage;
-  entry->magick=(MagickHandler *) IsDIB;
+  entry->decoder=(DecodeImageHandler *) ReadDIBImage;
+  entry->encoder=(EncodeImageHandler *) WriteDIBImage;
+  entry->magick=(IsImageFormatHandler *) IsDIB;
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
-  entry->description=
-    AcquireString("Microsoft Windows 3.X Packed Device-Independent Bitmap");
-  entry->module=AcquireString("DIB");
+  entry->description=ConstantString(
+    "Microsoft Windows 3.X Packed Device-Independent Bitmap");
+  entry->module=ConstantString("DIB");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -948,11 +956,10 @@ ModuleExport void UnregisterDIBImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
 {
@@ -965,12 +972,12 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register long
     i,
     x;
@@ -1000,14 +1007,15 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
   /*
     Initialize DIB raster file header.
   */
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image->colorspace != RGBColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   if (image->storage_class == DirectClass)
     {
       /*
         Full color DIB raster.
       */
       dib_info.number_colors=0;
-      dib_info.bits_per_pixel=image->matte ? 32 : 24;
+      dib_info.bits_per_pixel=(unsigned short) (image->matte ? 32 : 24);
     }
   else
     {
@@ -1015,16 +1023,20 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
         Colormapped DIB raster.
       */
       dib_info.bits_per_pixel=8;
-      if (IsMonochromeImage(image,&image->exception))
+      if (image_info->depth > 8)
+        dib_info.bits_per_pixel=16;
+      if (IsMonochromeImage(image,&image->exception) != MagickFalse)
         dib_info.bits_per_pixel=1;
-      dib_info.number_colors=1 << dib_info.bits_per_pixel;
+      dib_info.number_colors=(dib_info.bits_per_pixel == 16) ? 0 :
+        (1UL << dib_info.bits_per_pixel);
     }
   bytes_per_line=4*((image->columns*dib_info.bits_per_pixel+31)/32);
   dib_info.size=40;
   dib_info.width=(long) image->columns;
   dib_info.height=(long) image->rows;
   dib_info.planes=1;
-  dib_info.compression=0;
+  dib_info.compression=(unsigned long) (dib_info.bits_per_pixel == 16 ?
+    BI_BITFIELDS : BI_RGB);
   dib_info.image_size=bytes_per_line*image->rows;
   dib_info.x_pixels=75*39;
   dib_info.y_pixels=75*39;
@@ -1048,7 +1060,8 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
   /*
     Convert MIFF to DIB raster pixels.
   */
-  pixels=(unsigned char *) AcquireMagickMemory(dib_info.image_size);
+  pixels=(unsigned char *) AcquireQuantumMemory(dib_info.image_size,
+    sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   (void) ResetMagickMemory(pixels,0,dib_info.image_size);
@@ -1065,17 +1078,17 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
       */
       for (y=0; y < (long) image->rows; y++)
       {
-        p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+        p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
         if (p == (const PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=GetVirtualIndexQueue(image);
         q=pixels+(image->rows-y-1)*bytes_per_line;
         bit=0;
         byte=0;
         for (x=0; x < (long) image->columns; x++)
         {
           byte<<=1;
-          byte|=indexes[x] ? 0x01 : 0x00;
+          byte|=indexes[x] != 0 ? 0x01 : 0x00;
           bit++;
           if (bit == 8)
             {
@@ -1085,22 +1098,16 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
             }
            p++;
          }
-       if (bit != 0)
-         {
-           *q++=(unsigned char) (byte << (8-bit));
-           x++;
-         }
-       for (x=(long) (image->columns+7)/8; x < (long) bytes_per_line; x++)
-         *q++=0x00;
-       if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(y,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(SaveImageTag,y,image->rows,
-                image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+         if (bit != 0)
+           {
+             *q++=(unsigned char) (byte << (8-bit));
+             x++;
+           }
+        for (x=(long) (image->columns+7)/8; x < (long) bytes_per_line; x++)
+          *q++=0x00;
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
       }
       break;
     }
@@ -1111,24 +1118,49 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
       */
       for (y=0; y < (long) image->rows; y++)
       {
-        p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+        p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
         if (p == (const PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=GetVirtualIndexQueue(image);
         q=pixels+(image->rows-y-1)*bytes_per_line;
         for (x=0; x < (long) image->columns; x++)
-          *q++=indexes[x];
+          *q++=(unsigned char) indexes[x];
         for ( ; x < (long) bytes_per_line; x++)
           *q++=0x00;
-        if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(y,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(SaveImageTag,y,image->rows,
-                image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
+      }
+      break;
+    }
+    case 16:
+    {
+      unsigned short
+        word;
+      /*
+        Convert PseudoClass packet to DIB pixel. 
+      */
+      for (y=0; y < (long) image->rows; y++)
+      {
+        p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
+        if (p == (const PixelPacket *) NULL)
+          break;
+        q=pixels+(image->rows-y-1)*bytes_per_line;
+        for (x=0; x < (long) image->columns; x++)
+        {
+          word=(unsigned short) ((ScaleColor8to5((unsigned char)
+            ScaleQuantumToChar(p->red)) << 11) | (ScaleColor8to6((unsigned char)
+            ScaleQuantumToChar(p->green)) << 5) | (ScaleColor8to5(
+            (unsigned char) ScaleQuantumToChar((unsigned char) p->blue) << 0)));
+          *q++=(unsigned char)(word & 0xff);
+          *q++=(unsigned char)(word >> 8);
+          p++;
+        }
+        for (x=2L*image->columns; x < (long) bytes_per_line; x++)
+          *q++=0x00;
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
       }
       break;
     }
@@ -1140,7 +1172,7 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
       */
       for (y=0; y < (long) image->rows; y++)
       {
-        p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+        p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
         if (p == (const PixelPacket *) NULL)
           break;
         q=pixels+(image->rows-y-1)*bytes_per_line;
@@ -1154,23 +1186,17 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
           p++;
         }
         if (dib_info.bits_per_pixel == 24)
-          for (x=3*image->columns; x < (long) bytes_per_line; x++)
+          for (x=3L*image->columns; x < (long) bytes_per_line; x++)
             *q++=0x00;
-        if (image->previous == (Image *) NULL)
-          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-              (QuantumTick(y,image->rows) != MagickFalse))
-            {
-              status=image->progress_monitor(SaveImageTag,y,image->rows,
-                image->client_data);
-              if (status == MagickFalse)
-                break;
-            }
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
       }
       break;
     }
   }
   if (dib_info.bits_per_pixel == 8)
-    if (image->compression != NoCompression)
+    if (image_info->compression != NoCompression)
       {
         size_t
           length;
@@ -1178,25 +1204,27 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
         /*
           Convert run-length encoded raster pixels.
         */
-        length=2*(bytes_per_line+2)*(image->rows+2)+2;
-        dib_data=(unsigned char *) AcquireMagickMemory(length);
+        length=2UL*(bytes_per_line+2UL)+2UL;
+        dib_data=(unsigned char *) AcquireQuantumMemory(length,
+          (image->rows+2UL)*sizeof(*dib_data));
         if (pixels == (unsigned char *) NULL)
           {
             pixels=(unsigned char *) RelinquishMagickMemory(pixels);
             ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
           }
-        dib_info.image_size=EncodeImage(image,bytes_per_line,pixels,dib_data);
+        dib_info.image_size=(unsigned long) EncodeImage(image,bytes_per_line,
+          pixels,dib_data);
         pixels=(unsigned char *) RelinquishMagickMemory(pixels);
         pixels=dib_data;
-        dib_info.compression=1;
+        dib_info.compression = BI_RLE8;
       }
   /*
     Write DIB header.
   */
   (void) WriteBlobLSBLong(image,dib_info.size);
-  (void) WriteBlobLSBLong(image,dib_info.width);
-  (void) WriteBlobLSBLong(image,dib_info.height);
-  (void) WriteBlobLSBShort(image,dib_info.planes);
+  (void) WriteBlobLSBLong(image,(unsigned long) dib_info.width);
+  (void) WriteBlobLSBLong(image,(unsigned short) dib_info.height);
+  (void) WriteBlobLSBShort(image,(unsigned short) dib_info.planes);
   (void) WriteBlobLSBShort(image,dib_info.bits_per_pixel);
   (void) WriteBlobLSBLong(image,dib_info.compression);
   (void) WriteBlobLSBLong(image,dib_info.image_size);
@@ -1206,36 +1234,48 @@ static MagickBooleanType WriteDIBImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobLSBLong(image,dib_info.colors_important);
   if (image->storage_class == PseudoClass)
     {
-      unsigned char
-        *dib_colormap;
+      if (dib_info.bits_per_pixel <= 8)
+        {
+          unsigned char
+            *dib_colormap;
 
-      /*
-        Dump colormap to file.
-      */
-      dib_colormap=(unsigned char *)
-        AcquireMagickMemory((size_t) (4*(1 << dib_info.bits_per_pixel)));
-      if (dib_colormap == (unsigned char *) NULL)
-        ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-      q=dib_colormap;
-      for (i=0; i < (long) Min(image->colors,dib_info.number_colors); i++)
-      {
-        *q++=ScaleQuantumToChar(image->colormap[i].blue);
-        *q++=ScaleQuantumToChar(image->colormap[i].green);
-        *q++=ScaleQuantumToChar(image->colormap[i].red);
-        *q++=(Quantum) 0x0;
-      }
-      for ( ; i < (1L << dib_info.bits_per_pixel); i++)
-      {
-        *q++=(Quantum) 0x0;
-        *q++=(Quantum) 0x0;
-        *q++=(Quantum) 0x0;
-        *q++=(Quantum) 0x0;
-      }
-      (void) WriteBlob(image,4*(1 << dib_info.bits_per_pixel),dib_colormap);
-      dib_colormap=(unsigned char *) RelinquishMagickMemory(dib_colormap);
+          /*
+            Dump colormap to file.
+          */
+          dib_colormap=(unsigned char *) AcquireQuantumMemory((size_t)
+            (1UL << dib_info.bits_per_pixel),4*sizeof(dib_colormap));
+          if (dib_colormap == (unsigned char *) NULL)
+            ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+          q=dib_colormap;
+          for (i=0; i < (long) MagickMin(image->colors,dib_info.number_colors); i++)
+          {
+            *q++=ScaleQuantumToChar(image->colormap[i].blue);
+            *q++=ScaleQuantumToChar(image->colormap[i].green);
+            *q++=ScaleQuantumToChar(image->colormap[i].red);
+            *q++=(Quantum) 0x0;
+          }
+          for ( ; i < (long) (1L << dib_info.bits_per_pixel); i++)
+          {
+            *q++=(Quantum) 0x0;
+            *q++=(Quantum) 0x0;
+            *q++=(Quantum) 0x0;
+            *q++=(Quantum) 0x0;
+          }
+          (void) WriteBlob(image,(size_t) (4*(1 << dib_info.bits_per_pixel)),
+            dib_colormap);
+          dib_colormap=(unsigned char *) RelinquishMagickMemory(dib_colormap);
+        }
+      else
+        if ((dib_info.bits_per_pixel == 16) &&
+            (dib_info.compression == BI_BITFIELDS))
+          {
+            (void) WriteBlobLSBLong(image,0xf800);
+            (void) WriteBlobLSBLong(image,0x07e0);
+            (void) WriteBlobLSBLong(image,0x001f);
+          }
     }
   (void) WriteBlob(image,dib_info.image_size,pixels);
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
