@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -40,19 +40,34 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/attribute.h"
+#include "magick/property.h"
 #include "magick/blob.h"
-#include "magick/color_private.h"
+#include "magick/blob-private.h"
+#include "magick/cache.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/pixel-private.h"
+#include "magick/pixel-private.h"
 #include "magick/static.h"
+#include "magick/statistic.h"
 #include "magick/string_.h"
+#include "magick/module.h"
+#include "magick/module.h"
+
+/*
+  Forward declarations.
+*/
+#define FITSBlocksize  2880UL
 
 /*
   Forward declarations.
@@ -85,7 +100,6 @@ static MagickBooleanType
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsFITS(const unsigned char *magick,const size_t length)
 {
@@ -109,9 +123,9 @@ static MagickBooleanType IsFITS(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ReadFITSImage() reads a FITS image file and returns it.  It
-%  allocates the memory necessary for the new Image structure and returns a
-%  pointer to the new image.
+%  ReadFITSImage() reads a FITS image file and returns it.  It allocates the
+%  memory necessary for the new Image structure and returns a pointer to the
+%  new image.
 %
 %  The format of the ReadFITSImage method is:
 %
@@ -120,28 +134,100 @@ static MagickBooleanType IsFITS(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image: Method ReadFITSImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or if
-%      the image cannot be read.
-%
-%    o filename: Specifies the name of the image to read.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
+
+static inline double GetFITSPixel(Image *image,int bits_per_pixel)
+{
+  switch (image->depth >> 3)
+  {
+    case 1:
+      return((double) ReadBlobByte(image));
+    case 2:
+      return((double) ((short) ReadBlobShort(image)));
+    case 4:
+    {
+      if (bits_per_pixel > 0)
+        return((double) ((long) ReadBlobLong(image)));
+      return((double) ReadBlobFloat(image));
+    }
+    case 8:
+    {
+      if (bits_per_pixel > 0)
+        return((double) ((MagickOffsetType) ReadBlobLongLong(image)));
+    }
+    default:
+      break;
+  }
+  return(ReadBlobDouble(image));
+}
+
+static void GetFITSPixelExtrema(Image *image,const int bits_per_pixel,
+  double *minima,double *maxima)
+{
+  double
+    pixel;
+
+  MagickOffsetType
+    offset;
+
+  MagickSizeType
+    number_pixels;
+
+  register MagickOffsetType
+    i;
+
+  offset=TellBlob(image);
+  number_pixels=(MagickSizeType) image->columns*image->rows;
+  *minima=GetFITSPixel(image,bits_per_pixel);
+  *maxima=(*minima);
+  for (i=1; i < (MagickOffsetType) number_pixels; i++)
+  {
+    pixel=GetFITSPixel(image,bits_per_pixel);
+    if (pixel < *minima)
+      *minima=pixel;
+    if (pixel > *maxima)
+      *maxima=pixel;
+  }
+  (void) SeekBlob(image,offset,SEEK_SET);
+}
+
+static inline double GetFITSPixelRange(const unsigned long depth)
+{
+  return((double) ((MagickOffsetType) GetQuantumRange(depth)));
+}
+
+static void SetFITSUnsignedPixels(const size_t length,
+  const unsigned long bits_per_pixel,unsigned char *pixels)
+{
+  register long
+    i;
+
+  for (i=0; i < (long) length; i++)
+  {
+    *pixels^=0x80;
+    pixels+=bits_per_pixel >> 3;
+  }
+}
+
 static Image *ReadFITSImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
   typedef struct _FITSInfo
   {
+    EndianType
+      endian;
+
     int
       simple,
       bits_per_pixel,
       columns,
       rows,
       number_axes,
-      number_scenes;
+      number_planes;
 
     double
       min_data,
@@ -151,14 +237,12 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   } FITSInfo;
 
   char
-    keyword[MaxTextExtent],
-    value[MaxTextExtent];
+    keyword[FITSBlocksize],
+    value[FITSBlocksize];
 
   double
-    exponential[2048],
     pixel,
-    scale,
-    scaled_pixel;
+    scale;
 
   FITSInfo
     fits_info;
@@ -166,18 +250,10 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   Image
     *image;
 
-  IndexPacket
-    index;
-
   int
-    c,
-    quantum;
+    c;
 
   long
-    exponent,
-    j,
-    k,
-    l,
     scene,
     y;
 
@@ -188,69 +264,53 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   MagickSizeType
     number_pixels;
 
-  register IndexPacket
-    *indexes;
-
   register long
     x;
 
   register PixelPacket
     *q;
 
-  register long
-    i;
-
-  register unsigned char
-    *p;
-
-  ssize_t
-    count;
-
-  size_t
-    packet_size;
-
-  unsigned char
-    *fits_pixels,
-    long_quantum[8];
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
     Initialize image header.
   */
-  fits_info.simple=MagickFalse;
+  (void) ResetMagickMemory(&fits_info,0,sizeof(fits_info));
   fits_info.bits_per_pixel=8;
   fits_info.columns=1;
   fits_info.rows=1;
-  fits_info.number_scenes=1;
+  fits_info.rows=1;
+  fits_info.number_planes=1;
   fits_info.min_data=0.0;
   fits_info.max_data=0.0;
   fits_info.zero=0.0;
   fits_info.scale=1.0;
+  fits_info.endian=MSBEndian;
   /*
     Decode image header.
   */
   c=ReadBlobByte(image);
   if (c == EOF)
     {
-      DestroyImage(image);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
-  for ( ; ; )
+  for ( ; c != EOF; )
   {
     if (isalnum((int) ((unsigned char) c)) == 0)
       c=ReadBlobByte(image);
@@ -265,7 +325,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         p=keyword;
         do
         {
-          if ((size_t) (p-keyword) < (MaxTextExtent-1))
+          if ((size_t) (p-keyword) < (FITSBlocksize-1))
             *p++=(char) c;
           c=ReadBlobByte(image);
         } while ((isalnum((int) ((unsigned char) c)) != MagickFalse) ||
@@ -285,7 +345,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         p=value;
         while (isalnum(c) || (c == '-') || (c == '+') || (c == '.'))
         {
-          if ((size_t) (p-value) < (MaxTextExtent-1))
+          if ((size_t) (p-value) < (FITSBlocksize-1))
             *p++=c;
           c=ReadBlobByte(image);
         }
@@ -304,7 +364,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         if (LocaleCompare(keyword,"NAXIS2") == 0)
           fits_info.rows=atoi(value);
         if (LocaleCompare(keyword,"NAXIS3") == 0)
-          fits_info.number_scenes=atoi(value);
+          fits_info.number_planes=atoi(value);
         if (LocaleCompare(keyword,"DATAMAX") == 0)
           fits_info.max_data=atof(value);
         if (LocaleCompare(keyword,"DATAMIN") == 0)
@@ -313,13 +373,20 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
           fits_info.zero=atof(value);
         if (LocaleCompare(keyword,"BSCALE") == 0)
           fits_info.scale=atof(value);
-        (void) SetImageAttribute(image,keyword,value);
+        if (LocaleCompare(keyword,"XENDIAN") == 0)
+          {
+            if (LocaleCompare(keyword,"BIG") == 0)
+              fits_info.endian=MSBEndian;
+            else
+              fits_info.endian=LSBEndian;
+          }
+        (void) SetImageProperty(image,keyword,value);
       }
-    while ((TellBlob(image) % 80) != 0)
+    while (((TellBlob(image) % 80) != 0) && (c != EOF))
       c=ReadBlobByte(image);
     c=ReadBlobByte(image);
   }
-  while ((TellBlob(image) % 2880) != 0)
+  while (((TellBlob(image) % FITSBlocksize) != 0) && (c != EOF))
     c=ReadBlobByte(image);
   /*
     Verify that required image information is defined.
@@ -328,231 +395,57 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   if ((fits_info.simple == 0) || (fits_info.number_axes < 1) ||
       (fits_info.number_axes > 4) || (number_pixels == 0))
     ThrowReaderException(CorruptImageError,"ImageTypeNotSupported");
-  if (fits_info.bits_per_pixel == -32)
-    {
-      exponential[150]=1.0;
-      for (i=151; i < 256; i++)
-        exponential[i]=2.0*exponential[i-1];
-      for (i=149; i >= 0; i--)
-        exponential[i]=exponential[i+1]/2.0;
-    }
-  if (fits_info.bits_per_pixel == -64)
-    {
-      exponential[1075]=1.0;
-      for (i=1076; i < 2048; i++)
-        exponential[i]=2.0*exponential[i-1];
-      for (i=1074; i >= 0; i--)
-        exponential[i]=exponential[i+1]/2.0;
-    }
-  for (scene=0; scene < (long) fits_info.number_scenes; scene++)
+  for (scene=0; scene < (long) fits_info.number_planes; scene++)
   {
-    /*
-      Create linear colormap.
-    */
     image->columns=(unsigned long) fits_info.columns;
     image->rows=(unsigned long) fits_info.rows;
-    image->depth=fits_info.bits_per_pixel <= 8 ? 8UL : Min(QuantumDepth,16);
-    image->storage_class=PseudoClass;
+    image->depth=(unsigned long) (fits_info.bits_per_pixel < 0 ? -1 : 1)*
+      fits_info.bits_per_pixel;
+    image->endian=fits_info.endian;
     image->scene=(unsigned long) scene;
-    if (AllocateImageColormap(image,1UL << image->depth) == MagickFalse)
-      ThrowReaderException(ResourceLimitError,"UnableToAllocateColormap");
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     /*
       Initialize image structure.
     */
-    packet_size=(size_t) fits_info.bits_per_pixel/8;
-    if (fits_info.bits_per_pixel < 0)
-      packet_size=(size_t) (-fits_info.bits_per_pixel/8);
-    number_pixels=(MagickSizeType) image->columns*image->rows;
-    if ((packet_size*number_pixels) !=
-        (MagickSizeType) ((size_t) (packet_size*number_pixels)))
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    fits_pixels=(unsigned char *)
-      AcquireMagickMemory(packet_size*image->columns*image->rows);
-    if (fits_pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    /*
-      Convert FITS pixels to pixel packets.
-    */
-    count=ReadBlob(image,packet_size*image->columns*image->rows,fits_pixels);
-    if (count == 0)
-      ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
-    if ((fits_info.bits_per_pixel > 0) &&
-        (fits_info.min_data == 0.0) && (fits_info.max_data == 0.0))
-      fits_info.max_data=(1UL << fits_info.bits_per_pixel)-1;
-    else
+    if ((fits_info.min_data != 0.0) || (fits_info.max_data != 0.0))
       {
-        /*
-          Determine minimum and maximum intensity.
-        */
-        p=fits_pixels;
-        long_quantum[0]=(*p);
-        quantum=(*p++);
-        for (j=1; j <= (long) (packet_size-1); j++)
-        {
-          long_quantum[j]=(*p);
-          quantum=(quantum << 8) | (*p++);
-        }
-        pixel=(double) quantum;
-        if (fits_info.bits_per_pixel == -32)
-          {
-            j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
-               (long) long_quantum[3];
-            k=(int) *long_quantum;
-            exponent=((k & 127) << 1) | (j >> 23);
-            *(float *) long_quantum=
-              exponential[exponent]*(float) (j | 0x800000);
-            if ((exponent | j) == 0)
-              *(float *) long_quantum=0.0;
-            if (k & 128)
-              *(float *) long_quantum=(-(*(float *) long_quantum));
-            pixel=(double) (*((float *) long_quantum));
-          }
-        if (fits_info.bits_per_pixel == -64)
-          {
-            j=((long) long_quantum[1] << 24) |
-              ((long) long_quantum[2] << 16) |
-              ((long) long_quantum[3] << 8) |
-               (long) long_quantum[4];
-            k=(int) *long_quantum;
-            l=((int) long_quantum[5] << 16) | ((int) long_quantum[6] << 8) |
-               (int) long_quantum[7];
-            exponent=((k & 127) << 4) | (j >> 28);
-            *(double *) long_quantum=exponential[exponent]*(16777216.0*
-              (double) ((j & 0x0FFFFFFF) | 0x10000000)+(double) l);
-            if ((exponent | j | l) == 0)
-              *(double *) long_quantum=0.0;
-            if (k & 128)
-              *(double *)long_quantum=(-(*(double *) long_quantum));
-            pixel=(double) (*((double *) long_quantum));
-          }
-        fits_info.min_data=pixel*fits_info.scale-fits_info.zero;
-        fits_info.max_data=pixel*fits_info.scale-fits_info.zero;
-        for (i=1; i < (long) number_pixels; i++)
-        {
-          long_quantum[0]=(*p);
-          quantum=(*p++);
-          for (j=1; j <= (long) (packet_size-1); j++)
-          {
-            long_quantum[j]=(*p);
-            quantum=(quantum << 8) | (*p++);
-          }
-          pixel=(double) quantum;
-          if (fits_info.bits_per_pixel == -32)
-            {
-              j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
-                 (long) long_quantum[3];
-              k=(int) *long_quantum;
-              exponent=((k & 127) << 1) | (j >> 23);
-              *(float *) long_quantum=
-                exponential[exponent]*(float) (j | 0x800000);
-              if ((exponent | j) == 0)
-                *(float *) long_quantum=0.0;
-              if (k & 128)
-                *(float *) long_quantum=(-(*(float *) long_quantum));
-              pixel=(double) (*((float *) long_quantum));
-            }
-          if (fits_info.bits_per_pixel == -64)
-            {
-              j=((long) long_quantum[1] << 24) |
-                ((long) long_quantum[2] << 16) |
-                ((long) long_quantum[3] << 8) |
-                 (long) long_quantum[4];
-              k=(int) *long_quantum;
-              l=((int) long_quantum[5] << 16) | ((int) long_quantum[6] << 8) |
-                 (int) long_quantum[7];
-              exponent=((k & 127) << 4) | (j >> 28);
-              *(double *) long_quantum=exponential[exponent]*(16777216.0*
-                (double) ((j & 0x0FFFFFFF) | 0x10000000)+(double) l);
-              if ((exponent | j | l) == 0)
-                *(double *) long_quantum=0.0;
-              if (k & 128)
-                *(double *)long_quantum=(-(*(double *) long_quantum));
-              pixel=(double) (*((double *) long_quantum));
-            }
-          scaled_pixel=pixel*fits_info.scale-fits_info.zero;
-          if (scaled_pixel < fits_info.min_data)
-            fits_info.min_data=scaled_pixel;
-          if (scaled_pixel > fits_info.max_data)
-            fits_info.max_data=scaled_pixel;
-        }
+        if ((fits_info.bits_per_pixel != 0) && (fits_info.max_data == 0.0))
+          fits_info.max_data=GetFITSPixelRange((unsigned long)
+            fits_info.bits_per_pixel);
       }
+    else
+      GetFITSPixelExtrema(image,fits_info.bits_per_pixel,&fits_info.min_data,
+        &fits_info.max_data);
     /*
       Convert FITS pixels to pixel packets.
     */
-    scale=1.0;
-    if ((fits_info.bits_per_pixel < 0) || ((fits_info.max_data-
-        fits_info.min_data) > (double) ((1UL << image->depth)-1)))
-      scale=(double) ((1UL << image->depth)-1)/
-        (fits_info.max_data-fits_info.min_data);
-    p=fits_pixels;
+    scale=(double) QuantumRange/(fits_info.scale*(fits_info.max_data-
+      fits_info.min_data)+fits_info.zero);
     for (y=(long) image->rows-1; y >= 0; y--)
     {
-      q=SetImagePixels(image,0,y,image->columns,1);
+      q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         break;
-      indexes=GetIndexes(image);
       for (x=0; x < (long) image->columns; x++)
       {
-        long_quantum[0]=(*p);
-        quantum=(*p++);
-        for (j=1; j <= (long) (packet_size-1); j++)
-        {
-          long_quantum[j]=(*p);
-          quantum=(quantum << 8) | (*p++);
-        }
-        pixel=(double) quantum;
-        if (fits_info.bits_per_pixel == -32)
-          {
-            j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
-               (long) long_quantum[3];
-            k=(int) *long_quantum;
-            exponent=((k & 127) << 1) | (j >> 23);
-            *(float *) long_quantum=
-              exponential[exponent]*(float) (j | 0x800000);
-            if ((exponent | j) == 0)
-              *(float *) long_quantum=0.0;
-            if (k & 128)
-              *(float *) long_quantum=(-(*(float *) long_quantum));
-            pixel=(double) (*((float *) long_quantum));
-          }
-        if (fits_info.bits_per_pixel == -64)
-          {
-            j=((long) long_quantum[1] << 24) |
-              ((long) long_quantum[2] << 16) |
-              ((long) long_quantum[3] << 8) |
-               (long) long_quantum[4];
-            k=(long) *long_quantum;
-            l=((long) long_quantum[5] << 16) | ((long) long_quantum[6] << 8) |
-               (long) long_quantum[7];
-            exponent=(long) ((((unsigned long) k & 127) << 4) |
-              ((unsigned long) j >> 28));
-            *(double *) long_quantum=exponential[exponent]*(16777216.0*
-              (double) ((j & 0x0FFFFFFF) | 0x10000000)+(double) l);
-            if ((exponent | j | l) == 0)
-              *(double *) long_quantum=0.0;
-            if ((k & 128) != 0)
-              *(double *)long_quantum=(-(*(double *) long_quantum));
-            pixel=(double) (*((double *) long_quantum));
-          }
-        scaled_pixel=scale*
-          (pixel*fits_info.scale-fits_info.min_data-fits_info.zero);
-        index=(IndexPacket) ((scaled_pixel < 0.0) ? 0 :
-          ((unsigned long) scaled_pixel > ((1UL << image->depth)-1)) ?
-          ((1UL << image->depth)-1) : (unsigned long) (scaled_pixel+0.5));
-        index=ConstrainColormapIndex(image,index);
-        indexes[x]=index;
-        *q++=image->colormap[index];
+        pixel=GetFITSPixel(image,fits_info.bits_per_pixel);
+        q->red=(Quantum) RoundToQuantum(scale*(fits_info.scale*(pixel-
+          fits_info.min_data)+fits_info.zero));
+        q->green=q->red;
+        q->blue=q->red;
+        q++;
       }
-      if (SyncImagePixels(image) == MagickFalse)
+      if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
-      if (QuantumTick(y,image->rows) != MagickFalse)
-        if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-          break;
+      if (image->previous == (Image *) NULL)
+        {
+          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          if (status == MagickFalse)
+            break;
+        }
     }
-    fits_pixels=(unsigned char *) RelinquishMagickMemory(fits_pixels);
     if (EOFBlob(image) != MagickFalse)
       {
         ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
@@ -565,25 +458,25 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     if (image_info->number_scenes != 0)
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if (scene < (long) (fits_info.number_scenes-1))
+    if (scene < (long) (fits_info.number_planes-1))
       {
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        AcquireNextImage(image_info,image);
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
         if (status == MagickFalse)
           break;
       }
   }
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -607,22 +500,33 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterFITSImage method is:
 %
-%      RegisterFITSImage(void)
+%      unsigned long RegisterFITSImage(void)
 %
 */
-ModuleExport void RegisterFITSImage(void)
+ModuleExport unsigned long RegisterFITSImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("FITS");
-  entry->decoder=(DecoderHandler *) ReadFITSImage;
-  entry->encoder=(EncoderHandler *) WriteFITSImage;
-  entry->magick=(MagickHandler *) IsFITS;
+  entry->decoder=(DecodeImageHandler *) ReadFITSImage;
+  entry->encoder=(EncodeImageHandler *) WriteFITSImage;
+  entry->magick=(IsImageFormatHandler *) IsFITS;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Flexible Image Transport System");
-  entry->module=AcquireString("FITS");
+  entry->seekable_stream=MagickTrue;
+  entry->description=ConstantString("Flexible Image Transport System");
+  entry->module=ConstantString("FITS");
   (void) RegisterMagickInfo(entry);
+  entry=SetMagickInfo("FTS");
+  entry->decoder=(DecodeImageHandler *) ReadFITSImage;
+  entry->encoder=(EncodeImageHandler *) WriteFITSImage;
+  entry->magick=(IsImageFormatHandler *) IsFITS;
+  entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
+  entry->description=ConstantString("Flexible Image Transport System");
+  entry->module=ConstantString("FTS");
+  (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -647,6 +551,7 @@ ModuleExport void RegisterFITSImage(void)
 ModuleExport void UnregisterFITSImage(void)
 {
   (void) UnregisterMagickInfo("FITS");
+  (void) UnregisterMagickInfo("FTS");
 }
 
 /*
@@ -665,20 +570,21 @@ ModuleExport void UnregisterFITSImage(void)
 %
 %  The format of the WriteFITSImage method is:
 %
-%      MagickBooleanType WriteFITSImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
-%
 */
-static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
+  Image *image)
 {
   char
-    buffer[81],
+    header[FITSBlocksize],
     *fits_info;
 
   long
@@ -687,17 +593,18 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,Image *image
   MagickBooleanType
     status;
 
-  MagickSizeType
-    length;
+  QuantumInfo
+    *quantum_info;
 
   register const PixelPacket
     *p;
 
-  register long
-    i;
-
   size_t
-    packet_size;
+    length;
+
+  ssize_t
+    count,
+    offset;
 
   unsigned char
     *pixels;
@@ -710,70 +617,107 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,Image *image
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image->colorspace != RGBColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Allocate image memory.
   */
-  packet_size=(size_t) (image->depth > 8 ? 2 : 1);
-  fits_info=(char *) AcquireMagickMemory(2880*sizeof(*fits_info));
-  pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
-  if ((fits_info == (char *) NULL) || (pixels == (unsigned char *) NULL))
+  fits_info=(char *) AcquireQuantumMemory(FITSBlocksize,sizeof(*fits_info));
+  if (fits_info == (char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) ResetMagickMemory(fits_info,' ',FITSBlocksize*sizeof(*fits_info));
   /*
     Initialize image header.
   */
-  for (i=0; i < 2880; i++)
-    fits_info[i]=' ';
-  (void) strcpy(buffer,"SIMPLE  =                    T");
-  (void) strncpy(fits_info+0,buffer,strlen(buffer));
-  (void) FormatMagickString(buffer,MaxTextExtent,
-    "BITPIX  =                    %ld",image->depth);
-  (void) strncpy(fits_info+80,buffer,strlen(buffer));
-  (void) strcpy(buffer,"NAXIS   =                    2");
-  (void) strncpy(fits_info+160,buffer,strlen(buffer));
-  (void) FormatMagickString(buffer,MaxTextExtent,"NAXIS1  =           %10lu",
+  image->depth=GetImageQuantumDepth(image,MagickFalse);
+  quantum_info=AcquireQuantumInfo((const ImageInfo *) NULL,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+  offset=0;
+  (void) FormatMagickString(header,FITSBlocksize,
+    "SIMPLE  =                    T");
+  (void) strncpy(fits_info+offset,header,strlen(header));
+  offset+=80;
+  (void) FormatMagickString(header,FITSBlocksize,"BITPIX  =           %10ld",
+    (quantum_info->format == FloatingPointQuantumFormat ? -1 : 1)*image->depth);
+  (void) strncpy(fits_info+offset,header,strlen(header));
+  offset+=80;
+  (void) FormatMagickString(header,FITSBlocksize,"NAXIS   =           %10lu",
+    2UL);
+  (void) strncpy(fits_info+offset,header,strlen(header));
+  offset+=80;
+  (void) FormatMagickString(header,FITSBlocksize,"NAXIS1  =           %10lu",
     image->columns);
-  (void) strncpy(fits_info+240,buffer,strlen(buffer));
-  (void) FormatMagickString(buffer,MaxTextExtent,"NAXIS2  =           %10lu",
+  (void) strncpy(fits_info+offset,header,strlen(header));
+  offset+=80;
+  (void) FormatMagickString(header,FITSBlocksize,"NAXIS2  =           %10lu",
     image->rows);
-  (void) strncpy(fits_info+320,buffer,strlen(buffer));
-  (void) FormatMagickString(buffer,MaxTextExtent,"DATAMIN =           %10u",0);
-  (void) strncpy(fits_info+400,buffer,strlen(buffer));
-  (void) FormatMagickString(buffer,MaxTextExtent,"DATAMAX =           %10u",
-    (1 << image->depth)-1);
-  (void) strncpy(fits_info+480,buffer,strlen(buffer));
-  (void) strcpy(buffer,"HISTORY Created by ImageMagick.");
-  (void) strncpy(fits_info+560,buffer,strlen(buffer));
-  (void) strcpy(buffer,"END");
-  (void) strncpy(fits_info+640,buffer,strlen(buffer));
-  (void) WriteBlob(image,2880,(unsigned char *) fits_info);
+  (void) strncpy(fits_info+offset,header,strlen(header));
+  offset+=80;
+  (void) FormatMagickString(header,FITSBlocksize,"BSCALE  =         %E",1.0);
+  (void) strncpy(fits_info+offset,header,strlen(header));
+  offset+=80;
+  (void) FormatMagickString(header,FITSBlocksize,"BZERO   =         %E",
+    image->depth > 8 ? GetFITSPixelRange(image->depth) : 0.0);
+  (void) strncpy(fits_info+offset,header,strlen(header));
+  offset+=80;
+  (void) FormatMagickString(header,FITSBlocksize,"DATAMAX =         %E",
+    1.0*((MagickOffsetType) GetQuantumRange(image->depth)));
+  (void) strncpy(fits_info+offset,header,strlen(header));
+  offset+=80;
+  (void) FormatMagickString(header,FITSBlocksize,"DATAMIN =         %E",0.0);
+  (void) strncpy(fits_info+offset,header,strlen(header));
+  offset+=80;
+  if (image->endian == LSBEndian)
+    {
+      (void) FormatMagickString(header,FITSBlocksize,"XENDIAN = 'SMALL'");
+      (void) strncpy(fits_info+offset,header,strlen(header));
+      offset+=80;
+    }
+  (void) FormatMagickString(header,FITSBlocksize,"HISTORY %.72s",
+    GetMagickVersion((unsigned long *) NULL));
+  (void) strncpy(fits_info+offset,header,strlen(header));
+  offset+=80;
+  (void) strncpy(header,"END",FITSBlocksize);
+  (void) strncpy(fits_info+offset,header,strlen(header));
+  offset+=80;
+  (void) WriteBlob(image,FITSBlocksize,(unsigned char *) fits_info);
   /*
     Convert image to fits scale PseudoColor class.
   */
+  pixels=GetQuantumPixels(quantum_info);
+  length=GetQuantumExtent(image,quantum_info,GrayQuantum);
   for (y=(long) image->rows-1; y >= 0; y--)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) PopImagePixels(image,GrayQuantum,pixels);
-    (void) WriteBlob(image,packet_size*image->columns,pixels);
-    if (QuantumTick(image->rows-y-1,image->rows))
-      {
-        status=MagickMonitor(SaveImageTag,(MagickOffsetType) image->rows-y-1,
-          image->rows,&image->exception);
-        if (status == MagickFalse)
-          break;
-      }
+    length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+      GrayQuantum,pixels,&image->exception);
+    if (image->depth == 16)
+      SetFITSUnsignedPixels(image->columns,image->depth,pixels);
+    if (((image->depth == 32) || (image->depth == 64)) &&
+        (quantum_info->format != FloatingPointQuantumFormat))
+      SetFITSUnsignedPixels(image->columns,image->depth,pixels);
+    count=WriteBlob(image,length,pixels);
+    if (count != (ssize_t) length)
+      break;
+    status=SetImageProgress(image,SaveImageTag,y,image->rows);
+    if (status == MagickFalse)
+      break;
   }
-  (void) ResetMagickMemory(fits_info,0,2880*sizeof(*fits_info));
-  length=packet_size*image->columns*image->rows;
-  (void) WriteBlob(image,2880-(length % 2880),(unsigned char *) fits_info);
-  fits_info=(char *) RelinquishMagickMemory(fits_info);
-  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-  CloseBlob(image);
+  quantum_info=DestroyQuantumInfo(quantum_info);
+  length=(size_t) (FITSBlocksize-TellBlob(image) % FITSBlocksize);
+  if (length != 0)
+    {
+      (void) ResetMagickMemory(fits_info,0,length*sizeof(*fits_info));
+      (void) WriteBlob(image,length,(unsigned char *) fits_info);
+    }
+  fits_info=DestroyString(fits_info);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
