@@ -18,13 +18,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -41,17 +41,23 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/geometry.h"
+#include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/paint.h"
+#include "magick/property.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 #include "magick/utility.h"
 
 /*
@@ -107,8 +113,8 @@ static MagickBooleanType IsHTML(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RegisterHTMLImage() adds attributes for the HTML image format to
-%  the list of supported formats.  The attributes include the image format
+%  RegisterHTMLImage() adds properties for the HTML image format to
+%  the list of supported formats.  The properties include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
 %  whether the format supports native in-memory I/O, and a brief
@@ -116,38 +122,39 @@ static MagickBooleanType IsHTML(const unsigned char *magick,const size_t length)
 %
 %  The format of the RegisterHTMLImage method is:
 %
-%      RegisterHTMLImage(void)
+%      unsigned long RegisterHTMLImage(void)
 %
 */
-ModuleExport void RegisterHTMLImage(void)
+ModuleExport unsigned long RegisterHTMLImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("HTM");
-  entry->encoder=(EncoderHandler *) WriteHTMLImage;
-  entry->magick=(MagickHandler *) IsHTML;
+  entry->encoder=(EncodeImageHandler *) WriteHTMLImage;
+  entry->magick=(IsImageFormatHandler *) IsHTML;
   entry->adjoin=MagickFalse;
-  entry->description=
-    AcquireString("Hypertext Markup Language and a client-side image map");
-  entry->module=AcquireString("HTML");
+  entry->description=ConstantString(
+    "Hypertext Markup Language and a client-side image map");
+  entry->module=ConstantString("HTML");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("HTML");
-  entry->encoder=(EncoderHandler *) WriteHTMLImage;
-  entry->magick=(MagickHandler *) IsHTML;
+  entry->encoder=(EncodeImageHandler *) WriteHTMLImage;
+  entry->magick=(IsImageFormatHandler *) IsHTML;
   entry->adjoin=MagickFalse;
-  entry->description=
-    AcquireString("Hypertext Markup Language and a client-side image map");
-  entry->module=AcquireString("HTML");
+  entry->description=ConstantString(
+    "Hypertext Markup Language and a client-side image map");
+  entry->module=ConstantString("HTML");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("SHTML");
-  entry->encoder=(EncoderHandler *) WriteHTMLImage;
-  entry->magick=(MagickHandler *) IsHTML;
+  entry->encoder=(EncodeImageHandler *) WriteHTMLImage;
+  entry->magick=(IsImageFormatHandler *) IsHTML;
   entry->adjoin=MagickFalse;
-  entry->description=
-    AcquireString("Hypertext Markup Language and a client-side image map");
-  entry->module=AcquireString("HTML");
+  entry->description=ConstantString(
+    "Hypertext Markup Language and a client-side image map");
+  entry->module=ConstantString("HTML");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -195,13 +202,14 @@ ModuleExport void UnregisterHTMLImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
 %
 */
-static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,
+  Image *image)
 {
   char
     basename[MaxTextExtent],
@@ -233,12 +241,14 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  CloseBlob(image);
-  (void) SetImageColorspace(image,RGBColorspace);
+  (void) CloseBlob(image);
+  if (image->colorspace != RGBColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   *url='\0';
   if ((LocaleCompare(image_info->magick,"FTP") == 0) ||
       (LocaleCompare(image_info->magick,"HTTP") == 0))
@@ -272,8 +282,8 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
   status=MagickTrue;
   if (LocaleCompare(image_info->magick,"SHTML") != 0)
     {
-      const ImageAttribute
-        *attribute;
+      const char
+        *value;
 
       /*
         Open output image file.
@@ -284,12 +294,17 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
       /*
         Write the HTML image file.
       */
-      (void) WriteBlobString(image,"<html version=\"2.0\">\n");
+      (void) WriteBlobString(image,"<?xml version=\"1.0\" "
+        "encoding=\"US-ASCII\"?>\n");
+      (void) WriteBlobString(image,"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML "
+        "1.0 Strict//EN\" "
+        "\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n");
+      (void) WriteBlobString(image,"<html>\n");
       (void) WriteBlobString(image,"<head>\n");
-      attribute=GetImageAttribute(image,"label");
-      if (attribute != (const ImageAttribute *) NULL)
-        (void) FormatMagickString(buffer,MaxTextExtent,
-          "<title>%s</title>\n",attribute->value);
+      value=GetImageProperty(image,"label");
+      if (value != (const char *) NULL)
+        (void) FormatMagickString(buffer,MaxTextExtent,"<title>%s</title>\n",
+          value);
       else
         {
           GetPathComponent(filename,BasePath,basename);
@@ -298,17 +313,16 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
         }
       (void) WriteBlobString(image,buffer);
       (void) WriteBlobString(image,"</head>\n");
-      (void) WriteBlobString(image,"<body>\n");
-      (void) WriteBlobString(image,"<center>\n");
+      (void) WriteBlobString(image,"<body style=\"text-align: center;\">\n");
       (void) FormatMagickString(buffer,MaxTextExtent,"<h1>%s</h1>\n",
         image->filename);
       (void) WriteBlobString(image,buffer);
-      (void) WriteBlobString(image,"<br><br>\n");
+      (void) WriteBlobString(image,"<div>\n");
       (void) CopyMagickString(filename,image->filename,MaxTextExtent);
-      AppendImageFormat("gif",filename);
-      (void) FormatMagickString(buffer,MaxTextExtent,
-        "<img ismap usemap=\"#%s\" src=\"%s\" border=0>\n",
-        mapname,filename);
+      AppendImageFormat("png",filename);
+      (void) FormatMagickString(buffer,MaxTextExtent,"<img usemap=\"#%s\" "
+        "src=\"%s\" style=\"border: 0;\" alt=\"Image map\" />\n",mapname,
+        filename);
       (void) WriteBlobString(image,buffer);
       /*
         Determine the size and location of each image tile.
@@ -319,8 +333,8 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
       /*
         Write an image map.
       */
-      (void) FormatMagickString(buffer,MaxTextExtent,"<map name=\"%s\">\n",
-        mapname);
+      (void) FormatMagickString(buffer,MaxTextExtent,
+        "<map id=\"%s\" name=\"%s\">\n",mapname,mapname);
       (void) WriteBlobString(image,buffer);
       (void) FormatMagickString(buffer,MaxTextExtent,"  <area href=\"%s",
         url);
@@ -328,8 +342,8 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
       if (image->directory == (char *) NULL)
         {
           (void) FormatMagickString(buffer,MaxTextExtent,
-            "%s\" shape=rect coords=\"0,0,%lu,%lu\">\n",image->filename,
-            geometry.width-1,geometry.height-1);
+            "%s\" shape=\"rect\" coords=\"0,0,%lu,%lu\" alt=\"\" />\n",
+            image->filename,geometry.width-1,geometry.height-1);
           (void) WriteBlobString(image,buffer);
         }
       else
@@ -339,9 +353,9 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
           else
             {
               (void) FormatMagickString(buffer,MaxTextExtent,
-                "\" shape=rect coords=\"%ld,%ld,%ld,%ld\">\n",geometry.x,
-                geometry.y,geometry.x+(long) geometry.width-1,geometry.y+
-                (long) geometry.height-1);
+                "\" shape=\"rect\" coords=\"%ld,%ld,%ld,%ld\" alt=\"\" />\n",
+                geometry.x,geometry.y,geometry.x+(long) geometry.width-1,
+                geometry.y+(long) geometry.height-1);
               (void) WriteBlobString(image,buffer);
               if (*(p+1) != '\0')
                 {
@@ -350,29 +364,26 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
                   (void) WriteBlobString(image,buffer);
                 }
               geometry.x+=geometry.width;
-              if (geometry.x >= (long) image->columns)
+              if ((geometry.x+4) >= (long) image->columns)
                 {
                   geometry.x=0;
                   geometry.y+=geometry.height;
                 }
             }
       (void) WriteBlobString(image,"</map>\n");
-      if (image->montage != (char *) NULL)
-        (void) TransparentImage(image,GetOnePixel(image,0,0),
-          TransparentOpacity);
       (void) CopyMagickString(filename,image->filename,MaxTextExtent);
-      (void) WriteBlobString(image,"</center>\n");
+      (void) WriteBlobString(image,"</div>\n");
       (void) WriteBlobString(image,"</body>\n");
       (void) WriteBlobString(image,"</html>\n");
-      CloseBlob(image);
+      (void) CloseBlob(image);
       /*
-        Write the image as transparent GIF.
+        Write the image as PNG.
       */
       (void) CopyMagickString(image->filename,filename,MaxTextExtent);
-      AppendImageFormat("gif",image->filename);
-      next=image->next;
-      image->next=(Image *) NULL;
-      (void) strcpy(image->magick,"GIF");
+      AppendImageFormat("png",image->filename);
+      next=GetNextImageInList(image);
+      image->next=NewImageList();
+      (void) CopyMagickString(image->magick,"PNG",MaxTextExtent);
       (void) WriteImage(write_info,image);
       image->next=next;
       /*
@@ -388,7 +399,7 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
   status=OpenBlob(write_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  DestroyImageInfo(write_info);
+  write_info=DestroyImageInfo(write_info);
   /*
     Determine the size and location of each image tile.
   */
@@ -398,16 +409,16 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
   /*
     Write an image map.
   */
-  (void) FormatMagickString(buffer,MaxTextExtent,"<map name=\"%s\">\n",
-    mapname);
+  (void) FormatMagickString(buffer,MaxTextExtent,
+    "<map id=\"%s\" name=\"%s\">\n",mapname,mapname);
   (void) WriteBlobString(image,buffer);
   (void) FormatMagickString(buffer,MaxTextExtent,"  <area href=\"%s",url);
   (void) WriteBlobString(image,buffer);
   if (image->directory == (char *) NULL)
     {
       (void) FormatMagickString(buffer,MaxTextExtent,
-        "%s\" shape=rect coords=\"0,0,%lu,%lu\">\n",image->filename,
-        geometry.width-1,geometry.height-1);
+        "%s\" shape=\"rect\" coords=\"0,0,%lu,%lu\" alt=\"\" />\n",
+        image->filename,geometry.width-1,geometry.height-1);
       (void) WriteBlobString(image,buffer);
     }
   else
@@ -417,9 +428,9 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
       else
         {
           (void) FormatMagickString(buffer,MaxTextExtent,
-            "\" shape=rect coords=\"%ld,%ld,%ld,%ld\">\n",geometry.x,geometry.y,
-            geometry.x+(long) geometry.width-1,geometry.y+
-            (long) geometry.height-1);
+            "\" shape=\"rect\" coords=\"%ld,%ld,%ld,%ld\" alt=\"\" />\n",
+            geometry.x,geometry.y,geometry.x+(long) geometry.width-1,
+            geometry.y+(long) geometry.height-1);
           (void) WriteBlobString(image,buffer);
           if (*(p+1) != '\0')
             {
@@ -428,14 +439,14 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,Image *image
               (void) WriteBlobString(image,buffer);
             }
           geometry.x+=geometry.width;
-          if (geometry.x >= (long) image->columns)
+          if ((geometry.x+4) >= (long) image->columns)
             {
               geometry.x=0;
               geometry.y+=geometry.height;
             }
         }
   (void) WriteBlobString(image,"</map>\n");
-  CloseBlob(image);
+  (void) CloseBlob(image);
   (void) CopyMagickString(image->filename,filename,MaxTextExtent);
   return(status);
 }
