@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
@@ -52,9 +53,11 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 
 /*
   Forward declarations.
@@ -83,11 +86,10 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
@@ -129,7 +131,8 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *pixels;
 
   unsigned long
-    depth;
+    depth,
+    quantum;
 
   /*
     Open image file.
@@ -141,7 +144,7 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
@@ -154,15 +157,17 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Initialize image structure.
   */
   image->storage_class=PseudoClass;
-  status=AllocateImageColormap(image,(unsigned long)
+  status=AcquireImageColormap(image,(unsigned long)
     (image->offset != 0 ? image->offset : 256));
   if (status == MagickFalse)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   depth=GetImageQuantumDepth(image,MagickTrue);
   packet_size=(size_t) (depth/8);
-  pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
-  packet_size=(size_t) (image->colors > 256 ? 6 : 3);
-  colormap=(unsigned char *) AcquireMagickMemory(packet_size*image->colors);
+  pixels=(unsigned char *) AcquireQuantumMemory(image->columns,packet_size*
+    sizeof(*pixels));
+  packet_size=(size_t) (image->colors > 256 ? 6UL : 3UL);
+  colormap=(unsigned char *) AcquireQuantumMemory(image->colors,packet_size*
+    sizeof(*colormap));
   if ((pixels == (unsigned char *) NULL) ||
       (colormap == (unsigned char *) NULL))
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -183,17 +188,20 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   else
     for (i=0; i < (long) image->colors; i++)
     {
-      image->colormap[i].red=(Quantum) (*p++ << 8);
-      image->colormap[i].red|=(*p++);
-      image->colormap[i].green=(Quantum) (*p++ << 8);
-      image->colormap[i].green|=(*p++);
-      image->colormap[i].blue=(Quantum) (*p++ << 8);
-      image->colormap[i].blue|=(*p++);
+      quantum=(*p++ << 8);
+      quantum|=(*p++);
+      image->colormap[i].red=(Quantum) quantum;
+      quantum=(*p++ << 8);
+      quantum|=(*p++);
+      image->colormap[i].green=(Quantum) quantum;
+      quantum=(*p++ << 8);
+      quantum|=(*p++);
+      image->colormap[i].blue=(Quantum) quantum;
     }
   colormap=(unsigned char *) RelinquishMagickMemory(colormap);
   if (image_info->ping != MagickFalse)
     {
-      CloseBlob(image);
+      (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
   /*
@@ -203,10 +211,10 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   for (y=0; y < (long) image->rows; y++)
   {
     p=pixels;
-    q=SetImagePixels(image,0,y,image->columns,1);
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
+    indexes=GetAuthenticIndexQueue(image);
     count=ReadBlob(image,(size_t) packet_size*image->columns,pixels);
     if (count != (ssize_t) (packet_size*image->columns))
       break;
@@ -216,20 +224,20 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
       p++;
       if (image->colors > 256)
         {
-          index=ConstrainColormapIndex(image,((int) (index << 8)+(int) (*p)));
+          index=ConstrainColormapIndex(image,((unsigned long) index << 8)+(*p));
           p++;
         }
       indexes[x]=(IndexPacket) index;
-      *q++=image->colormap[index];
+      *q++=image->colormap[(long) index];
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
   }
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   if (y < (long) image->rows)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -253,23 +261,25 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterMAPImage method is:
 %
-%      RegisterMAPImage(void)
+%      unsigned long RegisterMAPImage(void)
 %
 */
-ModuleExport void RegisterMAPImage(void)
+ModuleExport unsigned long RegisterMAPImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("MAP");
-  entry->decoder=(DecoderHandler *) ReadMAPImage;
-  entry->encoder=(EncoderHandler *) WriteMAPImage;
+  entry->decoder=(DecodeImageHandler *) ReadMAPImage;
+  entry->encoder=(EncodeImageHandler *) WriteMAPImage;
   entry->adjoin=MagickFalse;
+  entry->format_type=ExplicitFormatType;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Colormap intensities and indices");
-  entry->module=AcquireString("MAP");
+  entry->description=ConstantString("Colormap intensities and indices");
+  entry->module=ConstantString("MAP");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -316,7 +326,7 @@ ModuleExport void UnregisterMAPImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
@@ -330,7 +340,7 @@ static MagickBooleanType WriteMAPImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
-  register IndexPacket
+  register const IndexPacket
     *indexes;
 
   register const PixelPacket
@@ -365,7 +375,8 @@ static MagickBooleanType WriteMAPImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image->colorspace != RGBColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Allocate colormap.
   */
@@ -373,9 +384,11 @@ static MagickBooleanType WriteMAPImage(const ImageInfo *image_info,Image *image)
     (void) SetImageType(image,PaletteType);
   depth=GetImageQuantumDepth(image,MagickTrue);
   packet_size=(size_t) (depth/8);
-  pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
-  packet_size=(size_t) (image->colors > 256 ? 6 : 3);
-  colormap=(unsigned char *) AcquireMagickMemory(packet_size*image->colors);
+  pixels=(unsigned char *) AcquireQuantumMemory(image->columns,packet_size*
+    sizeof(*pixels));
+  packet_size=(size_t) (image->colors > 256 ? 6UL : 3UL);
+  colormap=(unsigned char *) AcquireQuantumMemory(image->colors,packet_size*
+    sizeof(*colormap));
   if ((pixels == (unsigned char *) NULL) ||
       (colormap == (unsigned char *) NULL))
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
@@ -393,11 +406,11 @@ static MagickBooleanType WriteMAPImage(const ImageInfo *image_info,Image *image)
   else
     for (i=0; i < (long) image->colors; i++)
     {
-      *q++=(unsigned char) (image->colormap[i].red >> 8);
+      *q++=(unsigned char) ((unsigned long) image->colormap[i].red >> 8);
       *q++=(unsigned char) image->colormap[i].red;
-      *q++=(unsigned char) (image->colormap[i].green >> 8);
+      *q++=(unsigned char) ((unsigned long) image->colormap[i].green >> 8);
       *q++=(unsigned char) image->colormap[i].green;
-      *q++=(unsigned char) (image->colormap[i].blue >> 8);
+      *q++=(unsigned char) ((unsigned long) image->colormap[i].blue >> 8);
       *q++=(unsigned char) image->colormap[i].blue;
     }
   (void) WriteBlob(image,packet_size*image->colors,colormap);
@@ -407,20 +420,20 @@ static MagickBooleanType WriteMAPImage(const ImageInfo *image_info,Image *image)
   */
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
+    indexes=GetVirtualIndexQueue(image);
     q=pixels;
     for (x=0; x < (long) image->columns; x++)
     {
       if (image->colors > 256)
-        *q++=(unsigned char) (indexes[x] >> 8);
+        *q++=(unsigned char) ((unsigned long) indexes[x] >> 8);
       *q++=(unsigned char) indexes[x];
     }
     (void) WriteBlob(image,(size_t) (q-pixels),pixels);
   }
   pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(status);
 }
