@@ -10,14 +10,14 @@
 %                            P       CCCC  LLLLL                              %
 %                                                                             %
 %                                                                             %
-%                      Read/Write HP PCL Printer Format.                      %
+%                      Read/Write HP PCL Printer Format                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,9 +40,10 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/attribute.h"
+#include "magick/property.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
@@ -58,10 +59,13 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/profile.h"
 #include "magick/resource_.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 #include "magick/token.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
@@ -131,11 +135,10 @@ static MagickBooleanType IsPCL(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
@@ -149,7 +152,8 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     density[MaxTextExtent],
     filename[MaxTextExtent],
     geometry[MaxTextExtent],
-    options[MaxTextExtent];
+    options[MaxTextExtent],
+    input_filename[MaxTextExtent];
 
   const DelegateInfo
     *delegate_info;
@@ -198,13 +202,21 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Open image file.
   */
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  status=AcquireUniqueSymbolicLink(image_info->filename,input_filename);
+  if (status == MagickFalse)
+    {
+      ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
+        image_info->filename);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Set the page density.
   */
@@ -224,18 +236,17 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if ((flags & SigmaValue) == 0)
         image->y_resolution=image->x_resolution;
     }
-  (void) FormatMagickString(density,MaxTextExtent,"%gx%g",
-    image->x_resolution,image->y_resolution);
   /*
     Determine page geometry from the PCL media box.
   */
-  cmyk=MagickFalse;
+  cmyk=image->colorspace == CMYKColorspace ? MagickTrue : MagickFalse;
   count=0;
   (void) ResetMagickMemory(&bounding_box,0,sizeof(bounding_box));
   (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
   (void) ResetMagickMemory(&page,0,sizeof(page));
   (void) ResetMagickMemory(command,0,sizeof(command));
-  for (p=command; (c=ReadBlobByte(image)) != EOF; )
+  p=command;
+  for (c=ReadBlobByte(image); c != EOF; c=ReadBlobByte(image))
   {
     if (image_info->page != (char *) NULL)
       continue;
@@ -243,7 +254,8 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Note PCL elements.
     */
     *p++=(char) c;
-    if ((c != (int) '/') && ((size_t) (p-command) < (MaxTextExtent/2)))
+    if ((c != (int) '/') && (c != '\n') &&
+        ((size_t) (p-command) < (MaxTextExtent-1)))
       continue;
     *p='\0';
     p=command;
@@ -286,7 +298,7 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (height > page.height)
       page.height=height;
   }
-  CloseBlob(image);
+  (void) CloseBlob(image);
   /*
     Render PCL with the GhostPCL delegate.
   */
@@ -294,23 +306,29 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) ParseAbsoluteGeometry(PSPageGeometry,&page);
   if (image_info->page != (char *) NULL)
     (void) ParseAbsoluteGeometry(image_info->page,&page);
-  page.width=(unsigned long)
-    (page.width*image->x_resolution/delta.x+0.5);
-  page.height=(unsigned long)
-    (page.height*image->y_resolution/delta.y+0.5);
-  image=DestroyImage(image);
   (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu",
     page.width,page.height);
   if (image_info->monochrome != MagickFalse)
-    delegate_info=GetDelegateInfo("pcl-mono",(char *) NULL,exception);
+    delegate_info=GetDelegateInfo("pcl:mono",(char *) NULL,exception);
   else
      if (cmyk != MagickFalse)
-       delegate_info=GetDelegateInfo("pcl-cmyk",(char *) NULL,exception);
+       delegate_info=GetDelegateInfo("pcl:cmyk",(char *) NULL,exception);
      else
-       delegate_info=GetDelegateInfo("pcl-color",(char *) NULL,exception);
+       delegate_info=GetDelegateInfo("pcl:color",(char *) NULL,exception);
   if (delegate_info == (const DelegateInfo *) NULL)
     return((Image *) NULL);
   *options='\0';
+  if ((page.width == 0) || (page.height == 0))
+    (void) ParseAbsoluteGeometry(PSPageGeometry,&page);
+  if (image_info->page != (char *) NULL)
+    (void) ParseAbsoluteGeometry(image_info->page,&page);
+  (void) FormatMagickString(density,MaxTextExtent,"%gx%g",
+    image->x_resolution,image->y_resolution);
+  page.width=(unsigned long) (page.width*image->x_resolution/delta.x+0.5);
+  page.height=(unsigned long) (page.height*image->y_resolution/delta.y+0.5);
+  (void) FormatMagickString(options,MaxTextExtent,"-g%lux%lu ",
+    page.width,page.height);
+  image=DestroyImage(image);
   read_info=CloneImageInfo(image_info);
   *read_info->magick='\0';
   if (read_info->number_scenes != 0)
@@ -334,11 +352,13 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) FormatMagickString(command,MaxTextExtent,
     GetDelegateCommands(delegate_info),
     read_info->antialias != MagickFalse ? 4 : 1,
-    read_info->antialias != MagickFalse ? 4 : 1,geometry,density,options,
-    read_info->filename,image_info->filename);
-  status=(MagickBooleanType) SystemCommand(read_info->verbose,command);
+    read_info->antialias != MagickFalse ? 4 : 1,density,options,
+    read_info->filename,input_filename);
+  status=SystemCommand(read_info->verbose,command) != 0 ? MagickTrue :
+    MagickFalse;
   image=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(read_info->filename);
+  (void) RelinquishUniqueFileResource(input_filename);
   read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     ThrowReaderException(DelegateError,"PCLDelegateFailed");
@@ -385,22 +405,26 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterPCLImage method is:
 %
-%      RegisterPCLImage(void)
+%      unsigned long RegisterPCLImage(void)
 %
 */
-ModuleExport void RegisterPCLImage(void)
+ModuleExport unsigned long RegisterPCLImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("PCL");
-  entry->decoder=(DecoderHandler *) ReadPCLImage;
-  entry->encoder=(EncoderHandler *) WritePCLImage;
-  entry->magick=(MagickHandler *) IsPCL;
+  entry->decoder=(DecodeImageHandler *) ReadPCLImage;
+  entry->encoder=(EncodeImageHandler *) WritePCLImage;
+  entry->magick=(IsImageFormatHandler *) IsPCL;
   entry->adjoin=MagickFalse;
-  entry->description=AcquireString("Printer Control Language");
-  entry->module=AcquireString("PCL");
+  entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
+  entry->thread_support=EncoderThreadSupport;
+  entry->description=ConstantString("Printer Control Language");
+  entry->module=ConstantString("PCL");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -447,12 +471,10 @@ ModuleExport void UnregisterPCLImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
-%
-%
 */
 static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
 {
@@ -468,9 +490,6 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register long
     x;
 
@@ -492,7 +511,8 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image->colorspace != RGBColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Initialize the printer.
   */
@@ -507,11 +527,8 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
       (void) ParseGeometry(image_info->density,&geometry);
       density=(unsigned long) geometry.rho;
     }
-  if (IsMonochromeImage(image,&image->exception))
+  if (IsMonochromeImage(image,&image->exception) != MagickFalse)
     {
-      int
-        polarity;
-
       register unsigned char
         bit,
         byte;
@@ -519,20 +536,15 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
       /*
         Write PCL monochrome image.
       */
-      polarity=PixelIntensityToQuantum(&image->colormap[0]) < (QuantumRange/2);
-      if (image->colors == 2)
-        polarity=PixelIntensityToQuantum(&image->colormap[0]) <
-          PixelIntensityToQuantum(&image->colormap[1]);
       (void) FormatMagickString(buffer,MaxTextExtent,"\033*t%ldR",density);
       (void) WriteBlobString(image,buffer);
       (void) WriteBlobString(image,"\033*r1A");  /* start graphics */
       (void) WriteBlobString(image,"\033*b0M");  /* no compression */
       for (y=0; y < (long) image->rows; y++)
       {
-        p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+        p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
         if (p == (const PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
         bit=0;
         byte=0;
         (void) FormatMagickString(buffer,MaxTextExtent,"\033*b%luW",
@@ -541,7 +553,8 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
         for (x=0; x < (long) image->columns; x++)
         {
           byte<<=1;
-          byte|=indexes[x] == polarity ? 0x00 : 0x01;
+          if (PixelIntensity(p) < ((MagickRealType) QuantumRange/2.0))
+            byte|=0x01;
           bit++;
           if (bit == 8)
             {
@@ -549,17 +562,13 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
               bit=0;
               byte=0;
             }
+          p++;
         }
         if (bit != 0)
-          (void) WriteBlobByte(image,byte << (8-bit));
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(SaveImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+          (void) WriteBlobByte(image,(unsigned char) (byte << (8-bit)));
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
       }
       (void) WriteBlobString(image,"\033*rB");  /* end graphics */
     }
@@ -574,8 +583,8 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
       /*
         Allocate pixel buffers.
       */
-      pixels=(unsigned char *)
-        AcquireMagickMemory(3*image->columns*sizeof(unsigned char));
+      pixels=(unsigned char *) AcquireQuantumMemory(image->columns,
+        3UL*sizeof(*pixels));
       if (pixels == (unsigned char *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       /*
@@ -596,7 +605,7 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,"\033*b0M");  /* no compression */
       for (y=0; y < (long) image->rows; y++)
       {
-        p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+        p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
         if (p == (const PixelPacket *) NULL)
           break;
         q=pixels;
@@ -611,19 +620,14 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
           3*image->columns);  /* send row */
         (void) WriteBlobString(image,buffer);
         (void) WriteBlob(image,3*image->columns,pixels);
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(SaveImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,SaveImageTag,y,image->rows);
+        if (status == MagickFalse)
+          break;
       }
       (void) WriteBlobString(image,"\033*r0C");  /* end graphics */
       pixels=(unsigned char *) RelinquishMagickMemory(pixels);
     }
   (void) WriteBlobString(image,"\033E");
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
