@@ -10,14 +10,14 @@
 %                              T     GGG   A   A                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write Truevision Targa Image Format.                %
+%                    Read/Write Truevision Targa Image Format                 %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,9 +40,9 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
@@ -53,8 +53,12 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/property.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 
 /*
   Forward declarations.
@@ -83,11 +87,10 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
@@ -176,7 +179,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -207,16 +210,18 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     tga_info.colormap_size=(unsigned char) ReadBlobByte(image);
     tga_info.x_origin=ReadBlobLSBShort(image);
     tga_info.y_origin=ReadBlobLSBShort(image);
-    tga_info.width=ReadBlobLSBShort(image);
-    tga_info.height=ReadBlobLSBShort(image);
+    tga_info.width=(unsigned short) ReadBlobLSBShort(image);
+    tga_info.height=(unsigned short) ReadBlobLSBShort(image);
     tga_info.bits_per_pixel=(unsigned char) ReadBlobByte(image);
     tga_info.attributes=(unsigned char) ReadBlobByte(image);
+    if (EOFBlob(image) != MagickFalse)
+      ThrowReaderException(CorruptImageError,"UnableToReadImageData");
     /*
       Initialize image structure.
     */
-    image->matte=(tga_info.attributes & 0x0FU) != 0 ? MagickTrue : MagickFalse;
     image->columns=tga_info.width;
     image->rows=tga_info.height;
+    image->matte=tga_info.bits_per_pixel == 32 ? MagickTrue : MagickFalse;
     if ((tga_info.image_type != TGAColormap) &&
         (tga_info.image_type != TGARLEColormap))
       image->depth=(unsigned long) ((tga_info.bits_per_pixel <= 8) ? 8 :
@@ -244,7 +249,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         else
           {
             image->colors=0x01U << tga_info.bits_per_pixel;
-            if (AllocateImageColormap(image,image->colors) == MagickFalse)
+            if (AcquireImageColormap(image,image->colors) == MagickFalse)
               ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
       }
@@ -253,25 +258,32 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         char
           *comment;
 
+        size_t
+          length;
+
         /*
           TGA image comment.
         */
-        comment=(char *) AcquireMagickMemory(tga_info.id_length+MaxTextExtent);
+        length=(size_t) tga_info.id_length;
+        comment=(char *) NULL;
+        if (~length >= MaxTextExtent)
+          comment=(char *) AcquireQuantumMemory(length+MaxTextExtent,
+            sizeof(*comment));
         if (comment == (char *) NULL)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         count=ReadBlob(image,tga_info.id_length,(unsigned char *) comment);
         comment[tga_info.id_length]='\0';
-        (void) SetImageAttribute(image,"Comment",comment);
-        comment=(char *) RelinquishMagickMemory(comment);
+        (void) SetImageProperty(image,"comment",comment);
+        comment=DestroyString(comment);
       }
     (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
-    pixel.opacity=TransparentOpacity;
+    pixel.opacity=(Quantum) OpaqueOpacity;
     if (tga_info.colormap_type != 0)
       {
         /*
           Read TGA raster colormap.
         */
-        if (AllocateImageColormap(image,image->colors) == MagickFalse)
+        if (AcquireImageColormap(image,image->colors) == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         for (i=0; i < (long) image->colors; i++)
         {
@@ -283,7 +295,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 Gray scale.
               */
-              pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
               pixel.green=pixel.red;
               pixel.blue=pixel.red;
               break;
@@ -291,15 +303,19 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
             case 15:
             case 16:
             {
+              QuantumAny
+                range;
+
               /*
                 5 bits each of red green and blue.
               */
               j=(unsigned char) ReadBlobByte(image);
               k=(unsigned char) ReadBlobByte(image);
-              pixel.red=ScaleAnyToQuantum((int) (k & 0x7c) >> 2,31);
-              pixel.green=ScaleAnyToQuantum(((int) (k & 0x03) << 3)+
-                ((int) (j & 0xe0) >> 5),31);
-              pixel.blue=ScaleAnyToQuantum((int) (j & 0x1f),31);
+              range=GetQuantumRange(5UL);
+              pixel.red=ScaleAnyToQuantum(1UL*(k & 0x7c) >> 2,range);
+              pixel.green=ScaleAnyToQuantum((1UL*(k & 0x03) << 3)+
+                (1UL*(j & 0xe0) >> 5),range);
+              pixel.blue=ScaleAnyToQuantum(1UL*(j & 0x1f),range);
               break;
             }
             case 24:
@@ -308,9 +324,12 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 8 bits each of blue, green and red.
               */
-              pixel.blue=ScaleCharToQuantum(ReadBlobByte(image));
-              pixel.green=ScaleCharToQuantum(ReadBlobByte(image));
-              pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.blue=ScaleCharToQuantum((unsigned char)
+                ReadBlobByte(image));
+              pixel.green=ScaleCharToQuantum((unsigned char)
+                ReadBlobByte(image));
+              pixel.red=ScaleCharToQuantum((unsigned char)
+                ReadBlobByte(image));
               break;
             }
           }
@@ -327,7 +346,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     flag=0;
     skip=MagickFalse;
     real=0;
-    index=0;
+    index=(IndexPacket) 0;
     runlength=0;
     offset=0;
     for (y=0; y < (long) image->rows; y++)
@@ -335,10 +354,10 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       real=offset;
       if (((unsigned char) (tga_info.attributes & 0x20) >> 5) == 0)
         real=image->rows-real-1;
-      q=SetImagePixels(image,0,(long) real,image->columns,1);
+      q=QueueAuthenticPixels(image,0,(long) real,image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         break;
-      indexes=GetIndexes(image);
+      indexes=GetAuthenticIndexQueue(image);
       for (x=0; x < (long) image->columns; x++)
       {
         if ((tga_info.image_type == TGARLEColormap) ||
@@ -373,27 +392,35 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               */
               index=(IndexPacket) ReadBlobByte(image);
               if (tga_info.colormap_type != 0)
-                pixel=image->colormap[ConstrainColormapIndex(image,index)];
+                pixel=image->colormap[(long) ConstrainColormapIndex(image,
+                  1UL*index)];
               else
                 {
-                  pixel.red=ScaleCharToQuantum(index);
-                  pixel.green=ScaleCharToQuantum(index);
-                  pixel.blue=ScaleCharToQuantum(index);
+                  pixel.red=ScaleCharToQuantum((unsigned char) index);
+                  pixel.green=ScaleCharToQuantum((unsigned char) index);
+                  pixel.blue=ScaleCharToQuantum((unsigned char) index);
                 }
               break;
             }
             case 15:
             case 16:
             {
+              QuantumAny
+                range;
+
               /*
                 5 bits each of red green and blue.
               */
               j=(unsigned char) ReadBlobByte(image);
               k=(unsigned char) ReadBlobByte(image);
-              pixel.red=ScaleAnyToQuantum((int) (k & 0x7c) >> 2,31);
-              pixel.red=ScaleAnyToQuantum(((int) (k & 0x03) << 3)+
-                ((int) (j & 0xe0) >> 5),31);
-              pixel.blue=ScaleAnyToQuantum((int) (j & 0x1f),31);
+              range=GetQuantumRange(5UL);
+              pixel.red=ScaleAnyToQuantum(1UL*(k & 0x7c) >> 2,range);
+              pixel.green=ScaleAnyToQuantum((1UL*(k & 0x03) << 3)+
+                (1UL*(j & 0xe0) >> 5),range);
+              pixel.blue=ScaleAnyToQuantum(1UL*(j & 0x1f),range);
+              if (image->matte != MagickFalse)
+                pixel.opacity=(k & 0x80) != 0 ? (Quantum) OpaqueOpacity :
+                  (Quantum) TransparentOpacity; 
               if (image->storage_class == PseudoClass)
                 index=ConstrainColormapIndex(image,((unsigned long) k << 8)+j);
               break;
@@ -404,11 +431,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 8 bits each of blue green and red.
               */
-              pixel.blue=ScaleCharToQuantum(ReadBlobByte(image));
-              pixel.green=ScaleCharToQuantum(ReadBlobByte(image));
-              pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.blue=ScaleCharToQuantum((unsigned char)
+                ReadBlobByte(image));
+              pixel.green=ScaleCharToQuantum((unsigned char)
+                ReadBlobByte(image));
+              pixel.red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
               if (tga_info.bits_per_pixel == 32)
-                pixel.opacity=QuantumRange-ScaleCharToQuantum(ReadBlobByte(image));
+                pixel.opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(
+                  (unsigned char) ReadBlobByte(image)));
               break;
             }
           }
@@ -416,7 +446,12 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           ThrowReaderException(CorruptImageError,"UnableToReadImageData");
         if (image->storage_class == PseudoClass)
           indexes[x]=index;
-        *q++=pixel;
+        q->red=pixel.red;
+        q->green=pixel.green;
+        q->blue=pixel.blue;
+        if (image->matte != MagickFalse)
+          q->opacity=pixel.opacity;
+        q++;
       }
       if (((unsigned char) (tga_info.attributes & 0xc0) >> 6) == 4)
         offset+=4;
@@ -430,17 +465,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           base++;
           offset=base;
         }
-      if (SyncImagePixels(image) == MagickFalse)
+      if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(LoadImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        {
+          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          if (status == MagickFalse)
+            break;
+        }
     }
     if (EOFBlob(image) != MagickFalse)
       {
@@ -468,23 +500,20 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
+        AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        if (image->progress_monitor != (MagickProgressMonitor) NULL)
-          {
-            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
-              GetBlobSize(image),image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
+        if (status == MagickFalse)
+          break;
       }
   } while (status == MagickTrue);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -499,8 +528,8 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  RegisterTGAImage() adds attributes for the TGA image format to
-%  the list of supported formats.  The attributes include the image format
+%  RegisterTGAImage() adds properties for the TGA image format to
+%  the list of supported formats.  The properties include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
 %  whether the format supports native in-memory I/O, and a brief
@@ -508,38 +537,39 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterTGAImage method is:
 %
-%      RegisterTGAImage(void)
+%      unsigned long RegisterTGAImage(void)
 %
 */
-ModuleExport void RegisterTGAImage(void)
+ModuleExport unsigned long RegisterTGAImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("ICB");
-  entry->decoder=(DecoderHandler *) ReadTGAImage;
-  entry->encoder=(EncoderHandler *) WriteTGAImage;
-  entry->description=AcquireString("Truevision Targa image");
-  entry->module=AcquireString("TGA");
+  entry->decoder=(DecodeImageHandler *) ReadTGAImage;
+  entry->encoder=(EncodeImageHandler *) WriteTGAImage;
+  entry->description=ConstantString("Truevision Targa image");
+  entry->module=ConstantString("TGA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TGA");
-  entry->decoder=(DecoderHandler *) ReadTGAImage;
-  entry->encoder=(EncoderHandler *) WriteTGAImage;
-  entry->description=AcquireString("Truevision Targa image");
-  entry->module=AcquireString("TGA");
+  entry->decoder=(DecodeImageHandler *) ReadTGAImage;
+  entry->encoder=(EncodeImageHandler *) WriteTGAImage;
+  entry->description=ConstantString("Truevision Targa image");
+  entry->module=ConstantString("TGA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("VDA");
-  entry->decoder=(DecoderHandler *) ReadTGAImage;
-  entry->encoder=(EncoderHandler *) WriteTGAImage;
-  entry->description=AcquireString("Truevision Targa image");
-  entry->module=AcquireString("TGA");
+  entry->decoder=(DecodeImageHandler *) ReadTGAImage;
+  entry->encoder=(EncodeImageHandler *) WriteTGAImage;
+  entry->description=ConstantString("Truevision Targa image");
+  entry->module=ConstantString("TGA");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("VST");
-  entry->decoder=(DecoderHandler *) ReadTGAImage;
-  entry->encoder=(EncoderHandler *) WriteTGAImage;
-  entry->description=AcquireString("Truevision Targa image");
-  entry->module=AcquireString("TGA");
+  entry->decoder=(DecodeImageHandler *) ReadTGAImage;
+  entry->encoder=(EncodeImageHandler *) WriteTGAImage;
+  entry->description=ConstantString("Truevision Targa image");
+  entry->module=ConstantString("TGA");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -589,12 +619,19 @@ ModuleExport void UnregisterTGAImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
-%
 */
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
 {
 #define TargaColormap 1
@@ -629,8 +666,8 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
       attributes;
   } TargaInfo;
 
-  const ImageAttribute
-    *attribute;
+  const char
+    *value;
 
   long
     y;
@@ -641,12 +678,12 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
   MagickOffsetType
     scene;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register long
     x;
 
@@ -683,11 +720,14 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
     /*
       Initialize TGA raster file header.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if ((image->columns > 65535L) || (image->rows > 65535L))
+      ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
+    if (image->colorspace != RGBColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     targa_info.id_length=0;
-    attribute=GetImageAttribute(image,"Comment");
-    if (attribute != (const ImageAttribute *) NULL)
-      targa_info.id_length=(unsigned char) Min(strlen(attribute->value),255);
+    value=GetImageProperty(image,"comment");
+    if (value != (const char *) NULL)
+      targa_info.id_length=(unsigned char) MagickMin(strlen(value),255);
     targa_info.colormap_type=0;
     targa_info.colormap_index=0;
     targa_info.colormap_length=0;
@@ -745,7 +785,7 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(image,targa_info.attributes);
     if (targa_info.id_length != 0)
       (void) WriteBlob(image,targa_info.id_length,(unsigned char *)
-        attribute->value);
+        value);
     if (targa_info.image_type == TargaColormap)
       {
         unsigned char
@@ -754,8 +794,8 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
         /*
           Dump colormap to file (blue, green, red byte order).
         */
-        targa_colormap=(unsigned char *)
-          AcquireMagickMemory(3*targa_info.colormap_length);
+        targa_colormap=(unsigned char *) AcquireQuantumMemory((size_t)
+          targa_info.colormap_length,3UL*sizeof(*targa_colormap));
         if (targa_colormap == (unsigned char *) NULL)
           ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
         q=targa_colormap;
@@ -765,23 +805,25 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
           *q++=ScaleQuantumToChar(image->colormap[i].green);
           *q++=ScaleQuantumToChar(image->colormap[i].red);
         }
-        (void) WriteBlob(image,3*targa_info.colormap_length,targa_colormap);
+        (void) WriteBlob(image,(size_t) (3*targa_info.colormap_length),
+          targa_colormap);
         targa_colormap=(unsigned char *) RelinquishMagickMemory(targa_colormap);
       }
     /*
       Convert MIFF to TGA raster pixels.
     */
-    count=(ssize_t) ((targa_info.bits_per_pixel*targa_info.width) >> 3);
-    targa_pixels=(unsigned char *) AcquireMagickMemory((size_t) count);
+    count=(ssize_t) (targa_info.bits_per_pixel*targa_info.width)/8;
+    targa_pixels=(unsigned char *) AcquireQuantumMemory((size_t) count,
+      sizeof(*targa_pixels));
     if (targa_pixels == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     for (y=(long) (image->rows-1); y >= 0; y--)
     {
-      p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+      p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
       q=targa_pixels;
-      indexes=GetIndexes(image);
+      indexes=GetVirtualIndexQueue(image);
       for (x=0; x < (long) image->columns; x++)
       {
         if (targa_info.image_type == TargaColormap)
@@ -795,7 +837,8 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
               *q++=ScaleQuantumToChar(p->green);
               *q++=ScaleQuantumToChar(p->red);
               if (image->matte != MagickFalse)
-                *q++=QuantumRange-ScaleQuantumToChar(p->opacity);
+                *q++=(unsigned char) ScaleQuantumToChar((Quantum)
+                  (QuantumRange-p->opacity));
               if (image->colorspace == CMYKColorspace)
                 *q++=ScaleQuantumToChar(indexes[x]);
             }
@@ -803,28 +846,21 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
       }
       (void) WriteBlob(image,(size_t) (q-targa_pixels),targa_pixels);
       if (image->previous == (Image *) NULL)
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (QuantumTick(y,image->rows) != MagickFalse))
-          {
-            status=image->progress_monitor(SaveImageTag,y,image->rows,
-              image->client_data);
-            if (status == MagickFalse)
-              break;
-          }
+        {
+          status=SetImageProgress(image,SaveImageTag,y,image->rows);
+          if (status == MagickFalse)
+            break;
+        }
     }
     targa_pixels=(unsigned char *) RelinquishMagickMemory(targa_pixels);
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    if (image->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        status=image->progress_monitor(SaveImagesTag,scene,
-          GetImageListLength(image),image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
-    scene++;
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
+    if (status == MagickFalse)
+      break;
   } while (image_info->adjoin != MagickFalse);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
