@@ -10,20 +10,20 @@
 %                              Y     UUU     V                                %
 %                                                                             %
 %                                                                             %
-%            Read/Write Raw CCIR 601 4:1:1 or 4:2:2 Image Format.             %
+%            Read/Write Raw CCIR 601 4:1:1 or 4:2:2 Image Format              %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/www/Copyright.html                            %
+%    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -41,18 +41,25 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/colorspace.h"
 #include "magick/constitute.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception-private.h"
 #include "magick/geometry.h"
 #include "magick/image.h"
+#include "magick/image-private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/resize.h"
+#include "magick/quantum-private.h"
 #include "magick/static.h"
 #include "magick/string_.h"
+#include "magick/module.h"
 #include "magick/utility.h"
 
 /*
@@ -83,11 +90,10 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
@@ -108,14 +114,13 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     status;
 
   register const PixelPacket
-    *r;
+    *chroma_pixels;
 
   register long
     x;
 
   register PixelPacket
-    *q,
-    *s;
+    *q;
 
   register long
     i;
@@ -135,10 +140,11 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
-  image=AllocateImage(image_info);
+  image=AcquireImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
   image->depth=8;
@@ -177,20 +183,26 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
       if (status == MagickFalse)
         {
-          DestroyImageList(image);
+          image=DestroyImageList(image);
           return((Image *) NULL);
         }
       for (i=0; i < image->offset; i++)
-        (void) ReadBlobByte(image);
+        if (ReadBlobByte(image) == EOF)
+          {
+            ThrowFileException(exception,CorruptImageError,
+              "UnexpectedEndOfFile",image->filename);
+            break;
+          }
     }
   /*
     Allocate memory for a scanline.
   */
   if (interlace == NoInterlace)
-    scanline=(unsigned char *)
-      AcquireMagickMemory((size_t) (2*image->columns+2));
+    scanline=(unsigned char *) AcquireQuantumMemory((size_t) 2UL*
+      image->columns+2UL,sizeof(*scanline));
   else
-    scanline=(unsigned char *) AcquireMagickMemory((size_t) image->columns);
+    scanline=(unsigned char *) AcquireQuantumMemory((size_t) image->columns,
+      sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   do
@@ -211,45 +223,49 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
         if (status == MagickFalse)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
       }
     for (y=0; y < (long) image->rows; y++)
     {
+      register PixelPacket
+        *chroma_pixels;
+
       if (interlace == NoInterlace)
         {
-          if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,(size_t) (2*image->columns),scanline);
+          if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
+            count=ReadBlob(image,(size_t) (2*image->columns),scanline);
           p=scanline;
-          q=SetImagePixels(image,0,y,image->columns,1);
+          q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          s=SetImagePixels(chroma_image,0,y,chroma_image->columns,1);
-          if (s == (PixelPacket *) NULL)
+          chroma_pixels=QueueAuthenticPixels(chroma_image,0,y,
+            chroma_image->columns,1,exception);
+          if (chroma_pixels == (PixelPacket *) NULL)
             break;
           for (x=0; x < (long) image->columns; x+=2)
           {
-            s->red=0;
-            s->green=ScaleCharToQuantum(*p++);
+            chroma_pixels->red=(Quantum) 0;
+            chroma_pixels->green=ScaleCharToQuantum(*p++);
             q->red=ScaleCharToQuantum(*p++);
-            q->green=0;
-            q->blue=0;
+            q->green=(Quantum) 0;
+            q->blue=(Quantum) 0;
             q++;
             q->green=0;
             q->blue=0;
-            s->blue=ScaleCharToQuantum(*p++);
+            chroma_pixels->blue=ScaleCharToQuantum(*p++);
             q->red=ScaleCharToQuantum(*p++);
-            s++;
+            chroma_pixels++;
             q++;
           }
         }
       else
         {
-          if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,(size_t) image->columns,scanline);
+          if ((y > 0) || (GetPreviousImageInList(image) == (Image *) NULL))
+            count=ReadBlob(image,(size_t) image->columns,scanline);
           p=scanline;
-          q=SetImagePixels(image,0,y,image->columns,1);
+          q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
           for (x=0; x < (long) image->columns; x++)
@@ -260,23 +276,26 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q++;
           }
         }
-      if (SyncImagePixels(image) == MagickFalse)
+      if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
       if (interlace == NoInterlace)
-        if (SyncImagePixels(chroma_image) == MagickFalse)
+        if (SyncAuthenticPixels(chroma_image,exception) == MagickFalse)
           break;
       if (image->previous == (Image *) NULL)
-        if (MagickMonitor(LoadImageTag,y,image->rows,exception) == MagickFalse)
-          break;
+        {
+          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          if (status == MagickFalse)
+            break;
+        }
     }
     if (interlace == PartitionInterlace)
       {
-        CloseBlob(image);
+        (void) CloseBlob(image);
         AppendImageFormat("U",image->filename);
         status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
         if (status == MagickFalse)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
       }
@@ -284,37 +303,37 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       {
         for (y=0; y < (long) chroma_image->rows; y++)
         {
-          (void) ReadBlob(image,(size_t) chroma_image->columns,scanline);
+          count=ReadBlob(image,(size_t) chroma_image->columns,scanline);
           p=scanline;
-          q=SetImagePixels(chroma_image,0,y,chroma_image->columns,1);
+          q=QueueAuthenticPixels(chroma_image,0,y,chroma_image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
           for (x=0; x < (long) chroma_image->columns; x++)
           {
-            q->red=0;
+            q->red=(Quantum) 0;
             q->green=ScaleCharToQuantum(*p++);
-            q->blue=0;
+            q->blue=(Quantum) 0;
             q++;
           }
-          if (SyncImagePixels(chroma_image) == MagickFalse)
+          if (SyncAuthenticPixels(chroma_image,exception) == MagickFalse)
             break;
         }
       if (interlace == PartitionInterlace)
         {
-          CloseBlob(image);
+          (void) CloseBlob(image);
           AppendImageFormat("V",image->filename);
           status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
           if (status == MagickFalse)
             {
-              DestroyImageList(image);
+              image=DestroyImageList(image);
               return((Image *) NULL);
             }
         }
       for (y=0; y < (long) chroma_image->rows; y++)
       {
-        (void) ReadBlob(image,(size_t) chroma_image->columns,scanline);
+        count=ReadBlob(image,(size_t) chroma_image->columns,scanline);
         p=scanline;
-        q=GetImagePixels(chroma_image,0,y,chroma_image->columns,1);
+        q=GetAuthenticPixels(chroma_image,0,y,chroma_image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
         for (x=0; x < (long) chroma_image->columns; x++)
@@ -322,7 +341,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q->blue=ScaleCharToQuantum(*p++);
           q++;
         }
-        if (SyncImagePixels(chroma_image) == MagickFalse)
+        if (SyncAuthenticPixels(chroma_image,exception) == MagickFalse)
           break;
       }
     }
@@ -331,29 +350,29 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     resize_image=ResizeImage(chroma_image,image->columns,image->rows,
       TriangleFilter,1.0,exception);
-    DestroyImage(chroma_image);
+    chroma_image=DestroyImage(chroma_image);
     if (resize_image == (Image *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     for (y=0; y < (long) image->rows; y++)
     {
-      q=GetImagePixels(image,0,y,image->columns,1);
-      r=AcquireImagePixels(resize_image,0,y,resize_image->columns,1,
+      q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
+      chroma_pixels=GetVirtualPixels(resize_image,0,y,resize_image->columns,1,
         &resize_image->exception);
-      if ((q == (PixelPacket *) NULL) || (r == (const PixelPacket *) NULL))
+      if ((q == (PixelPacket *) NULL) ||
+          (chroma_pixels == (const PixelPacket *) NULL))
         break;
       for (x=0; x < (long) image->columns; x++)
       {
-        q->green=r->green;
-        q->blue=r->blue;
-        r++;
+        q->green=chroma_pixels->green;
+        q->blue=chroma_pixels->blue;
+        chroma_pixels++;
         q++;
       }
-      if (SyncImagePixels(image) == MagickFalse)
+      if (SyncAuthenticPixels(image,exception) == MagickFalse)
         break;
     }
-    DestroyImage(resize_image);
+    resize_image=DestroyImage(resize_image);
     image->colorspace=YCbCrColorspace;
-    (void) SetImageColorspace(image,RGBColorspace);
     if (interlace == PartitionInterlace)
       (void) CopyMagickString(image->filename,image_info->filename,
         MaxTextExtent);
@@ -378,21 +397,21 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Allocate next image structure.
         */
-        AllocateNextImage(image_info,image);
-        if (image->next == (Image *) NULL)
+        AcquireNextImage(image_info,image);
+        if (GetNextImageInList(image) == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
+        status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
+          GetBlobSize(image));
         if (status == MagickFalse)
           break;
       }
   } while (count != 0);
   scanline=(unsigned char *) RelinquishMagickMemory(scanline);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -416,22 +435,23 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterYUVImage method is:
 %
-%      RegisterYUVImage(void)
+%      unsigned long RegisterYUVImage(void)
 %
 */
-ModuleExport void RegisterYUVImage(void)
+ModuleExport unsigned long RegisterYUVImage(void)
 {
   MagickInfo
     *entry;
 
   entry=SetMagickInfo("YUV");
-  entry->decoder=(DecoderHandler *) ReadYUVImage;
-  entry->encoder=(EncoderHandler *) WriteYUVImage;
+  entry->decoder=(DecodeImageHandler *) ReadYUVImage;
+  entry->encoder=(EncodeImageHandler *) WriteYUVImage;
   entry->adjoin=MagickFalse;
   entry->raw=MagickTrue;
-  entry->description=AcquireString("CCIR 601 4:1:1 or 4:2:2");
-  entry->module=AcquireString("YUV");
+  entry->description=ConstantString("CCIR 601 4:1:1 or 4:2:2");
+  entry->module=ConstantString("YUV");
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -479,11 +499,10 @@ ModuleExport void UnregisterYUVImage(void)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
-%
 */
 static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
 {
@@ -521,8 +540,8 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
-  interlace=image_info->interlace;
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  interlace=image->interlace;
   horizontal_factor=2;
   vertical_factor=2;
   if (image_info->sampling_factor != (char *) NULL)
@@ -572,7 +591,6 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
       Sample image to an even width and height, if necessary.
     */
     image->depth=8;
-    (void) SetImageColorspace(image,RGBColorspace);
     width=image->columns+(image->columns & (horizontal_factor-1));
     height=image->rows+(image->rows & (vertical_factor-1));
     yuv_image=ResizeImage(image,width,height,TriangleFilter,1.0,
@@ -595,11 +613,11 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
         */
         for (y=0; y < (long) yuv_image->rows; y++)
         {
-          p=AcquireImagePixels(yuv_image,0,y,yuv_image->columns,1,
+          p=GetVirtualPixels(yuv_image,0,y,yuv_image->columns,1,
             &yuv_image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          s=AcquireImagePixels(chroma_image,0,y,chroma_image->columns,1,
+          s=GetVirtualPixels(chroma_image,0,y,chroma_image->columns,1,
             &chroma_image->exception);
           if (s == (const PixelPacket *) NULL)
             break;
@@ -615,15 +633,13 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             x++;
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
-              {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
-                if (status == MagickFalse)
-                  break;
-              }
+            {
+              status=SetImageProgress(image,SaveImageTag,y,image->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
-        DestroyImage(yuv_image);
+        yuv_image=DestroyImage(yuv_image);
       }
     else
       {
@@ -632,7 +648,7 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
         */
         for (y=0; y < (long) yuv_image->rows; y++)
         {
-          p=AcquireImagePixels(yuv_image,0,y,yuv_image->columns,1,
+          p=GetVirtualPixels(yuv_image,0,y,yuv_image->columns,1,
             &yuv_image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
@@ -642,21 +658,25 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             p++;
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows) != MagickFalse)
-              {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
-                if (status == MagickFalse)
-                  break;
-              }
+            {
+              status=SetImageProgress(image,SaveImageTag,y,image->rows);
+              if (status == MagickFalse)
+                break;
+            }
         }
-        DestroyImage(yuv_image);
+        yuv_image=DestroyImage(yuv_image);
+        if (image->previous == (Image *) NULL)
+          {
+            status=SetImageProgress(image,SaveImageTag,1,3);
+            if (status == MagickFalse)
+              break;
+          }
         /*
           Initialize U channel.
         */
         if (interlace == PartitionInterlace)
           {
-            CloseBlob(image);
+            (void) CloseBlob(image);
             AppendImageFormat("U",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
@@ -665,7 +685,7 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
           }
         for (y=0; y < (long) chroma_image->rows; y++)
         {
-          p=AcquireImagePixels(chroma_image,0,y,chroma_image->columns,1,
+          p=GetVirtualPixels(chroma_image,0,y,chroma_image->columns,1,
             &chroma_image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
@@ -675,12 +695,18 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             p++;
           }
         }
+        if (image->previous == (Image *) NULL)
+          {
+            status=SetImageProgress(image,SaveImageTag,2,3);
+            if (status == MagickFalse)
+              break;
+          }
         /*
           Initialize V channel.
         */
         if (interlace == PartitionInterlace)
           {
-            CloseBlob(image);
+            (void) CloseBlob(image);
             AppendImageFormat("V",image->filename);
             status=OpenBlob(image_info,image,WriteBinaryBlobMode,
               &image->exception);
@@ -689,7 +715,7 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
           }
         for (y=0; y < (long) chroma_image->rows; y++)
         {
-          p=AcquireImagePixels(chroma_image,0,y,chroma_image->columns,1,
+          p=GetVirtualPixels(chroma_image,0,y,chroma_image->columns,1,
             &chroma_image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
@@ -699,19 +725,25 @@ static MagickBooleanType WriteYUVImage(const ImageInfo *image_info,Image *image)
             p++;
           }
         }
+        if (image->previous == (Image *) NULL)
+          {
+            status=SetImageProgress(image,SaveImageTag,2,3);
+            if (status == MagickFalse)
+              break;
+          }
       }
-    DestroyImage(chroma_image);
+    chroma_image=DestroyImage(chroma_image);
     if (interlace == PartitionInterlace)
       (void) CopyMagickString(image->filename,image_info->filename,
         MaxTextExtent);
-    if (image->next == (Image *) NULL)
+    if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
+    status=SetImageProgress(image,SaveImagesTag,scene++,
+      GetImageListLength(image));
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
-  CloseBlob(image);
+  (void) CloseBlob(image);
   return(MagickTrue);
 }
