@@ -10,14 +10,14 @@
 %           CCCC   OOO   M   M  P      R  R   EEEEE   SSSSS  SSSSS            %
 %                                                                             %
 %                                                                             %
-%                  Image Compression/Decompression Methods                    %
+%             MagickCore Image Compression/Decompression Methods              %
 %                                                                             %
 %                           Software Design                                   %
 %                             John Cristy                                     %
 %                              May  1993                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -44,6 +44,7 @@
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/color-private.h"
+#include "magick/cache.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
@@ -52,10 +53,12 @@
 #include "magick/list.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/monitor-private.h"
+#include "magick/option.h"
 #include "magick/resource_.h"
 #include "magick/string_.h"
-#if defined(HasTIFF)
-#if defined(HAVE_TIFFCONF_H)
+#if defined(MAGICKCORE_TIFF_DELEGATE)
+#if defined(MAGICKCORE_HAVE_TIFFCONF_H)
 #include "tiffconf.h"
 #endif
 #include "tiffio.h"
@@ -63,14 +66,14 @@
 #else
 #define CCITTParam  "0"
 #endif
-#if defined(HasZLIB)
+#if defined(MAGICKCORE_ZLIB_DELEGATE)
 #include "zlib.h"
 #endif
 
 /*
   Typedef declarations.
 */
-typedef struct _Ascii85Info
+struct _Ascii85Info
 {
   long
     offset,
@@ -78,7 +81,7 @@ typedef struct _Ascii85Info
 
   unsigned char
     buffer[10];
-} Ascii85Info;
+};
 
 typedef struct HuffmanTable
 {
@@ -273,8 +276,7 @@ MagickExport void Ascii85Initialize(Image *image)
   if (image->ascii85 == (Ascii85Info *) NULL)
     image->ascii85=(Ascii85Info *) AcquireMagickMemory(sizeof(*image->ascii85));
   if (image->ascii85 == (Ascii85Info *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      image->filename);
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   (void) ResetMagickMemory(image->ascii85,0,sizeof(*image->ascii85));
   image->ascii85->line_break=MaxLineExtent << 1;
   image->ascii85->offset=0;
@@ -362,10 +364,24 @@ MagickExport void Ascii85Encode(Image *image,const unsigned char code)
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
-%
+%    o image: the image.
 %
 */
+
+static inline size_t MagickMax(const size_t x,const size_t y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
+static inline size_t MagickMin(const size_t x,const size_t y)
+{
+  if (x < y)
+    return(x);
+  return(y);
+}
+
 MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
 {
 #define HashSize  1021
@@ -403,6 +419,9 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
   const HuffmanTable
     *entry;
 
+  ExceptionInfo
+    *exception;
+
   HuffmanTable
     **mb_hash,
     **mw_hash;
@@ -414,11 +433,10 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
     byte;
 
   long
-    count,
     y;
 
   MagickBooleanType
-    status;
+    proceed;
 
   register IndexPacket
     *indexes;
@@ -433,6 +451,9 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
   register unsigned char
     *p;
 
+  ssize_t
+    count;
+
   unsigned char
     *scanline;
 
@@ -455,9 +476,10 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  mb_hash=(HuffmanTable **) AcquireMagickMemory(HashSize*sizeof(*mb_hash));
-  mw_hash=(HuffmanTable **) AcquireMagickMemory(HashSize*sizeof(*mw_hash));
-  scanline=(unsigned char *) AcquireMagickMemory((size_t) image->columns);
+  mb_hash=(HuffmanTable **) AcquireQuantumMemory(HashSize,sizeof(*mb_hash));
+  mw_hash=(HuffmanTable **) AcquireQuantumMemory(HashSize,sizeof(*mw_hash));
+  scanline=(unsigned char *) AcquireQuantumMemory((size_t) image->columns,
+    sizeof(*scanline));
   if ((mb_hash == (HuffmanTable **) NULL) ||
       (mw_hash == (HuffmanTable **) NULL) ||
       (scanline == (unsigned char *) NULL))
@@ -490,6 +512,7 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
   image->x_resolution=204.0;
   image->y_resolution=196.0;
   image->units=PixelsPerInchResolution;
+  exception=(&image->exception);
   for (y=0; ((y < (long) image->rows) && (null_lines < 3)); )
   {
     /*
@@ -570,7 +593,7 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
         {
           count+=entry->count;
           if ((x+count) > (long) image->columns)
-            count=(long) image->columns-x;
+            count=(ssize_t) image->columns-x;
           if (count > 0)
             {
               if (color != MagickFalse)
@@ -603,29 +626,24 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
       Transfer scanline to image pixels.
     */
     p=scanline;
-    q=SetImagePixels(image,0,y,image->columns,1);
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
+    indexes=GetAuthenticIndexQueue(image);
     for (x=0; x < (long) image->columns; x++)
     {
       index=(IndexPacket) (*p++);
       indexes[x]=index;
-      *q++=image->colormap[index];
+      *q++=image->colormap[(long) index];
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
+      break;
+    proceed=SetImageProgress(image,LoadImageTag,y,image->rows);
+    if (proceed == MagickFalse)
       break;
-    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-        (QuantumTick(y,image->rows) != MagickFalse))
-      {
-        status=image->progress_monitor(LoadImageTag,y,image->rows,
-          image->client_data);
-        if (status == MagickFalse)
-          break;
-      }
     y++;
   }
-  image->rows=(unsigned long) Max(y-3,1);
+  image->rows=(unsigned long) MagickMax((size_t) y-3,1);
   image->compression=FaxCompression;
   /*
     Free decoder memory.
@@ -656,9 +674,9 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info..
+%    o image_info: the image info..
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
@@ -693,6 +711,9 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
   const HuffmanTable
     *entry;
 
+  ExceptionInfo
+    *exception;
+
   int
     k,
     runlength;
@@ -704,14 +725,8 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
   Image
     *huffman_image;
 
-  IndexPacket
-    polarity;
-
   MagickBooleanType
-    status;
-
-  register IndexPacket
-    *indexes;
+    proceed;
 
   register long
     i,
@@ -741,14 +756,19 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   width=image->columns;
   if (LocaleCompare(image_info->magick,"FAX") == 0)
-    width=Max(image->columns,1728);
-  scanline=(unsigned char *) AcquireMagickMemory((size_t) width+1);
+    width=(unsigned long) MagickMax(image->columns,1728);
+  scanline=(unsigned char *) AcquireQuantumMemory((size_t) width+1UL,
+    sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
+  (void) ResetMagickMemory(scanline,0,width*sizeof(*scanline));
   huffman_image=CloneImage(image,0,0,MagickTrue,&image->exception);
   if (huffman_image == (Image *) NULL)
-    return(MagickFalse);
+    {
+      scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+      return(MagickFalse);
+    }
   (void) SetImageType(huffman_image,BilevelType);
   byte='\0';
   bit=(unsigned char) 0x80;
@@ -764,30 +784,20 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
       OutputBit(1);
     }
   /*
-    Compress runlength encoded to 1D Huffman pixels.
+    Compress to 1D Huffman pixels.
   */
-  polarity=(IndexPacket) (PixelIntensityToQuantum(
-    &huffman_image->colormap[0]) >= (QuantumRange/2) ? 0x00 : 0x01);
-  if (huffman_image->colors == 2)
-    polarity=(IndexPacket)
-      (PixelIntensityToQuantum(&huffman_image->colormap[0]) <
-       PixelIntensityToQuantum(&huffman_image->colormap[1]) ? 0x00 : 0x01);
-  q=scanline;
-  for (i=0; i < (long) width; i++)
-    *q++=(unsigned char) polarity;
+  exception=(&huffman_image->exception);
   q=scanline;
   for (y=0; y < (long) huffman_image->rows; y++)
   {
-    p=AcquireImagePixels(huffman_image,0,y,huffman_image->columns,1,
-      &huffman_image->exception);
+    p=GetVirtualPixels(huffman_image,0,y,huffman_image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(huffman_image);
     for (x=0; x < (long) huffman_image->columns; x++)
     {
-      *q=(unsigned char)
-        (indexes[x] == polarity ? polarity == 0 : polarity != 0);
-      q++;
+      *q++=(unsigned char) (PixelIntensity(p) >= ((MagickRealType)
+        QuantumRange/2.0) ? 0 : 1);
+      p++;
     }
     /*
       Huffman encode scanline.
@@ -798,7 +808,7 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
       /*
         Output white run.
       */
-      for (runlength=0; ((n > 0) && ((Quantum) *q == polarity)); n--)
+      for (runlength=0; ((n > 0) && (*q == 0)); n--)
       {
         q++;
         runlength++;
@@ -808,18 +818,18 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
           if (runlength < 1792)
             entry=MWTable+((runlength/64)-1);
           else
-            entry=EXTable+(Min(runlength,2560)-1792)/64;
+            entry=EXTable+(MagickMin((size_t) runlength,2560)-1792)/64;
           runlength-=entry->count;
           HuffmanOutputCode(entry);
         }
-      entry=TWTable+Min(runlength,63);
+      entry=TWTable+MagickMin((size_t) runlength,63);
       HuffmanOutputCode(entry);
       if (n != 0)
         {
           /*
             Output black run.
           */
-          for (runlength=0; (((Quantum) *q != polarity) && (n > 0)); n--)
+          for (runlength=0; ((*q != 0) && (n > 0)); n--)
           {
             q++;
             runlength++;
@@ -828,11 +838,11 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
             {
               entry=MBTable+((runlength/64)-1);
               if (runlength >= 1792)
-                entry=EXTable+(Min(runlength,2560)-1792)/64;
+                entry=EXTable+(MagickMin((size_t) runlength,2560)-1792)/64;
               runlength-=entry->count;
               HuffmanOutputCode(entry);
             }
-          entry=TBTable+Min(runlength,63);
+          entry=TBTable+MagickMin((size_t) runlength,63);
           HuffmanOutputCode(entry);
         }
     }
@@ -844,14 +854,12 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
     OutputBit(1);
     q=scanline;
     if (GetPreviousImageInList(huffman_image) == (Image *) NULL)
-      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-          (QuantumTick(y,huffman_image->rows) != MagickFalse))
-        {
-          status=image->progress_monitor(LoadImageTag,y,huffman_image->rows,
-            image->client_data);
-          if (status == MagickFalse)
-            break;
-        }
+      {
+        proceed=SetImageProgress(huffman_image,LoadImageTag,y,
+          huffman_image->rows);
+        if (proceed == MagickFalse)
+          break;
+      }
   }
   /*
     End of page.
@@ -879,7 +887,7 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
   return(MagickTrue);
 }
 
-#if defined(HasTIFF)
+#if defined(MAGICKCORE_TIFF_DELEGATE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -901,9 +909,9 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info..
+%    o image_info: the image info..
 %
-%    o image: The image.
+%    o image: the image.
 %
 */
 MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
@@ -912,6 +920,9 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   char
     filename[MaxTextExtent];
 
+  FILE
+    *file;
+
   Image
     *huffman_image;
 
@@ -921,9 +932,6 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   int
     unique_file;
 
-  long
-    j;
-
   MagickBooleanType
     status;
 
@@ -939,13 +947,13 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   uint16
     fillorder;
 
-  unsigned char
-    *buffer;
-
-  unsigned long
+  uint32
     *byte_count,
     strip_size;
 
+  unsigned char
+    *buffer;
+
   /*
     Write image as CCITTFax4 TIFF image to a temporary file.
   */
@@ -961,12 +969,23 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   DestroyBlob(huffman_image);
   huffman_image->blob=CloneBlobInfo((BlobInfo *) NULL);
   (void) SetImageType(huffman_image,BilevelType);
+  file=(FILE *) NULL;
   unique_file=AcquireUniqueFileResource(filename);
+  if (unique_file != -1)
+    file=fdopen(unique_file,"wb"); 
+  if ((unique_file == -1) || (file == (FILE *) NULL))
+    {
+      (void) CopyMagickString(image->filename,filename,MaxTextExtent);
+      ThrowFileException(&image->exception,FileOpenError,
+        "UnableToCreateTemporaryFile",image->filename);
+      return(MagickFalse);
+    }
   (void) FormatMagickString(huffman_image->filename,MaxTextExtent,"tiff:%s",
     filename);
   write_info=CloneImageInfo(image_info);
-  SetImageInfoFile(write_info,fdopen(unique_file,"wb"));
+  SetImageInfoFile(write_info,file);
   write_info->compression=Group4Compression;
+  (void) SetImageOption(write_info,"quantum:polarity","min-is-white");
   status=WriteImage(write_info,huffman_image);
   write_info=DestroyImageInfo(write_info);
   if (status == MagickFalse)
@@ -974,6 +993,8 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   tiff=TIFFOpen(filename,"rb");
   if (tiff == (TIFF *) NULL)
     {
+      huffman_image=DestroyImage(huffman_image);
+      (void) fclose(file);
       (void) RelinquishUniqueFileResource(filename);
       ThrowFileException(&image->exception,FileOpenError,"UnableToOpenFile",
         image_info->filename);
@@ -988,10 +1009,13 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   for (i=1; i < (long) TIFFNumberOfStrips(tiff); i++)
     if (byte_count[i] > strip_size)
       strip_size=byte_count[i];
-  buffer=(unsigned char *) AcquireMagickMemory((size_t) strip_size);
+  buffer=(unsigned char *) AcquireQuantumMemory((size_t) strip_size,
+    sizeof(*buffer));
   if (buffer == (unsigned char *) NULL)
     {
       TIFFClose(tiff);
+      huffman_image=DestroyImage(huffman_image);
+      (void) fclose(file);
       (void) RelinquishUniqueFileResource(filename);
       ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
         image_info->filename);
@@ -1007,12 +1031,12 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
       byte_count[i]);
     if (fillorder == FILLORDER_LSB2MSB)
       TIFFReverseBits(buffer,(unsigned long) count);
-    for (j=0; j < (long) count; j++)
-      (void) WriteBlobByte(image,(unsigned char) buffer[j]);
+    (void) WriteBlob(image,count,buffer);
   }
   buffer=(unsigned char *) RelinquishMagickMemory(buffer);
   TIFFClose(tiff);
   huffman_image=DestroyImage(huffman_image);
+  (void) fclose(file);
   (void) RelinquishUniqueFileResource(filename);
   return(MagickTrue);
 }
@@ -1024,158 +1048,10 @@ MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  ThrowBinaryException(MissingDelegateError,"TIFFLibraryIsNotAvailable",
-    image->filename);
-}
-#endif
-
-#if defined(HasJPEG)
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   J P E G E n c o d e I m a g e                                             %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  JPEGEncodeImage() compresses an image via JPEG compression.
-%
-%  The format of the JPEGEncodeImage method is:
-%
-%      MagickBooleanType JPEGEncodeImage(const ImageInfo *image_info,
-%        Image *image)
-%
-%  A description of each parameter follows:
-%
-%    o image_info: The image info..
-%
-%    o image: The image.
-%
-*/
-MagickExport MagickBooleanType JPEGEncodeImage(const ImageInfo *image_info,
-  Image *image)
-{
-  Image
-    *jpeg_image;
-
-  ImageInfo
-    *jpeg_info;
-
-  size_t
-    length;
-
-  unsigned char
-    *blob;
-
-  /*
-    Write image as JPEG image to a temporary file.
-  */
-  jpeg_image=CloneImage(image,0,0,MagickTrue,&image->exception);
-  if (jpeg_image == (Image *) NULL)
-    ThrowWriterException(CoderError,image->exception.reason);
-  DestroyBlob(jpeg_image);
-  jpeg_image->blob=CloneBlobInfo((BlobInfo *) NULL);
-  jpeg_info=CloneImageInfo(image_info);
-  *jpeg_info->filename='\0';
-  (void) strcpy(jpeg_info->magick,"JPEG");
-  length=0;
-  blob=ImageToBlob(jpeg_info,jpeg_image,&length,&image->exception);
-  jpeg_info=DestroyImageInfo(jpeg_info);
-  if (blob == (void *) NULL)
-    ThrowWriterException(CoderError,image->exception.reason);
-  (void) WriteBlob(image,length,blob);
-  jpeg_image=DestroyImage(jpeg_image);
-  blob=(unsigned char *) RelinquishMagickMemory(blob);
-  return(MagickTrue);
-}
-#else
-MagickExport MagickBooleanType JPEGEncodeImage(const ImageInfo *image_info,
-  Image *image)
-{
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  ThrowBinaryException(MissingDelegateError,"JPEGLibraryIsNotAvailable",
-    image->filename);
-}
-#endif
-
-#if defined(HasJP2)
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   J P 2 E n c o d e I m a g e                                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  JP2EncodeImage() compresses an image via JPEG-2000 compression.
-%
-%  The format of the JP2EncodeImage method is:
-%
-%      MagickBooleanType JP2EncodeImage(const ImageInfo *image_info,
-%        Image *image)
-%
-%  A description of each parameter follows:
-%
-%    o image_info: The image info..
-%
-%    o image: The image.
-%
-*/
-MagickExport MagickBooleanType JP2EncodeImage(const ImageInfo *image_info,
-  Image *image)
-{
-  Image
-    *jp2_image;
-
-  ImageInfo
-    *jp2_info;
-
-  size_t
-    length;
-
-  unsigned char
-    *blob;
-
-  /*
-    Write image as JP2 image to a temporary file.
-  */
-  jp2_image=CloneImage(image,0,0,MagickTrue,&image->exception);
-  if (jp2_image == (Image *) NULL)
-    ThrowWriterException(CoderError,image->exception.reason);
-  DestroyBlob(jp2_image);
-  jp2_image->blob=CloneBlobInfo((BlobInfo *) NULL);
-  jp2_info=CloneImageInfo(image_info);
-  *jp2_info->filename='\0';
-  (void) strcpy(jp2_info->magick,"JP2");
-  length=0;
-  blob=ImageToBlob(jp2_info,jp2_image,&length,&image->exception);
-  jp2_info=DestroyImageInfo(jp2_info);
-  if (blob == (void *) NULL)
-    ThrowWriterException(CoderError,image->exception.reason);
-  (void) WriteBlob(image,length,blob);
-  jp2_image=DestroyImage(jp2_image);
-  blob=(unsigned char *) RelinquishMagickMemory(blob);
-  return(MagickTrue);
-}
-#else
-MagickExport MagickBooleanType JP2EncodeImage(
-  const ImageInfo *magick_unused(image_info),Image *image)
-{
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  ThrowBinaryException(MissingDelegateError,"JP2LibraryIsNotAvailable",
+  (void) ThrowMagickException(&image->exception,GetMagickModule(),
+    MissingDelegateError,"DelegateLibrarySupportNotBuiltIn","`%s' (TIFF)",
     image->filename);
+  return(MagickFalse);
 }
 #endif
 
@@ -1200,14 +1076,13 @@ MagickExport MagickBooleanType JP2EncodeImage(
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o length:  A value that specifies the number of pixels to compress.
 %
-%    o pixels: The address of an unsigned array of characters containing the
+%    o pixels: the address of an unsigned array of characters containing the
 %      pixels to compress.
 %
-%
 */
 MagickExport MagickBooleanType LZWEncodeImage(Image *image,const size_t length,
   unsigned char *pixels)
@@ -1258,7 +1133,7 @@ MagickExport MagickBooleanType LZWEncodeImage(Image *image,const size_t length,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (unsigned char *) NULL);
-  table=(TableType *) AcquireMagickMemory((1 << 12)*sizeof(*table));
+  table=(TableType *) AcquireQuantumMemory(1UL << 12,sizeof(*table));
   if (table == (TableType *) NULL)
     return(MagickFalse);
   /*
@@ -1363,14 +1238,13 @@ MagickExport MagickBooleanType LZWEncodeImage(Image *image,const size_t length,
 %
 %  A description of each parameter follows:
 %
-%    o image: The image.
+%    o image: the image.
 %
 %    o length:  A value that specifies the number of pixels to compress.
 %
-%    o pixels: The address of an unsigned array of characters containing the
+%    o pixels: the address of an unsigned array of characters containing the
 %      pixels to compress.
 %
-%
 */
 MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
   const size_t length,unsigned char *pixels)
@@ -1393,7 +1267,7 @@ MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(pixels != (unsigned char *) NULL);
-  packbits=(unsigned char *) AcquireMagickMemory(128);
+  packbits=(unsigned char *) AcquireQuantumMemory(128UL,sizeof(*packbits));
   if (packbits == (unsigned char *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
@@ -1478,7 +1352,7 @@ MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
   return(MagickTrue);
 }
 
-#if defined(HasZLIB)
+#if defined(MAGICKCORE_ZLIB_DELEGATE)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -1496,24 +1370,21 @@ MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
 %  The format of the ZLIBEncodeImage method is:
 %
 %      MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
-%        const unsigned long quality,unsigned char *pixels)
+%        unsigned char *pixels)
 %
 %  A description of each parameter follows:
 %
-%    o file: The address of a structure of type FILE.  ZLIB encoded pixels
+%    o file: the address of a structure of type FILE.  ZLIB encoded pixels
 %      are written to this file.
 %
 %    o length:  A value that specifies the number of pixels to compress.
 %
-%    o quality: the compression level (0-100).
-%
-%    o pixels: The address of an unsigned array of characters containing the
+%    o pixels: the address of an unsigned array of characters containing the
 %      pixels to compress.
 %
-%
 */
 MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
-  const unsigned long quality,unsigned char *pixels)
+  unsigned char *pixels)
 {
   int
     status;
@@ -1522,10 +1393,10 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
     i;
 
   size_t
-    compressed_packets;
+    compress_packets;
 
   unsigned char
-    *compressed_pixels;
+    *compress_pixels;
 
   z_stream
     stream;
@@ -1534,19 +1405,21 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  compressed_packets=(size_t) (1.001*length+12);
-  compressed_pixels=(unsigned char *) AcquireMagickMemory(compressed_packets);
-  if (compressed_pixels == (unsigned char *) NULL)
+  compress_packets=(size_t) (1.001*length+12);
+  compress_pixels=(unsigned char *) AcquireQuantumMemory(compress_packets,
+    sizeof(*compress_pixels));
+  if (compress_pixels == (unsigned char *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   stream.next_in=pixels;
   stream.avail_in=(unsigned int) length;
-  stream.next_out=compressed_pixels;
-  stream.avail_out=(unsigned int) compressed_packets;
+  stream.next_out=compress_pixels;
+  stream.avail_out=(unsigned int) compress_packets;
   stream.zalloc=(alloc_func) NULL;
   stream.zfree=(free_func) NULL;
   stream.opaque=(voidpf) NULL;
-  status=deflateInit(&stream,(int) Min(quality/10,9));
+  status=deflateInit(&stream,(int) (image->quality ==
+    UndefinedCompressionQuality ? 7 : MagickMin(image->quality/10,9)));
   if (status == Z_OK)
     {
       status=deflate(&stream,Z_FINISH);
@@ -1554,26 +1427,27 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
         status=deflateEnd(&stream);
       else
         (void) deflateEnd(&stream);
-      compressed_packets=(size_t) stream.total_out;
+      compress_packets=(size_t) stream.total_out;
     }
   if (status != Z_OK)
     ThrowBinaryException(CoderError,"UnableToZipCompressImage",image->filename)
   else
-    for (i=0; i < (long) compressed_packets; i++)
-      (void) WriteBlobByte(image,compressed_pixels[i]);
-  compressed_pixels=(unsigned char *) RelinquishMagickMemory(compressed_pixels);
-  return((MagickBooleanType) (status == Z_OK));
+    for (i=0; i < (long) compress_packets; i++)
+      (void) WriteBlobByte(image,compress_pixels[i]);
+  compress_pixels=(unsigned char *) RelinquishMagickMemory(compress_pixels);
+  return(status == Z_OK ? MagickTrue : MagickFalse);
 }
 #else
 MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,
-  const size_t magick_unused(length),const unsigned long magick_unused(quality),
-  unsigned char *magick_unused(pixels))
+  const size_t magick_unused(length),unsigned char *magick_unused(pixels))
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  ThrowBinaryException(MissingDelegateError,"ZLIBLibraryIsNotAvailable",
+  (void) ThrowMagickException(&image->exception,GetMagickModule(),
+    MissingDelegateError,"DelegateLibrarySupportNotBuiltIn","`%s' (ZIP)",
     image->filename);
+  return(MagickFalse);
 }
 #endif
