@@ -9,14 +9,14 @@
 %                    M   M  A   A   GGGG  IIIII   CCCC                        %
 %                                                                             %
 %                                                                             %
-%                     ImageMagick Image Magic Methods                         %
+%                      MagickCore Image Magic Methods                         %
 %                                                                             %
 %                              Software Design                                %
 %                              Bob Friesenhahn                                %
 %                                 July 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -51,6 +51,7 @@
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
+#include "magick/xml-tree.h"
 
 /*
   Define declarations.
@@ -78,12 +79,15 @@ static const char
     "  <magic name=\"CGM\" offset=\"0\" target=\"BEGMF\" />"
     "  <magic name=\"DCM\" offset=\"128\" target=\"DICM\" />"
     "  <magic name=\"DCX\" offset=\"0\" target=\"\\261\\150\\336\\72\" />"
+    "  <magic name=\"DDS\" offset=\"0\" target=\"DDS \" />"
     "  <magic name=\"DIB\" offset=\"0\" target=\"\\050\\000\" />"
+    "  <magic name=\"DJVU\" offset=\"0\" target=\"AT&TFORM\" />"
     "  <magic name=\"DOT\" offset=\"0\" target=\"digraph\" />"
     "  <magic name=\"DPX\" offset=\"0\" target=\"SDPX\" />"
     "  <magic name=\"DPX\" offset=\"0\" target=\"XPDS\" />"
     "  <magic name=\"EMF\" offset=\"40\" target=\"\\040\\105\\115\\106\\000\\000\\001\\000\" />"
     "  <magic name=\"EPT\" offset=\"0\" target=\"\\305\\320\\323\\306\" />"
+    "  <magic name=\"EXR\" offset=\"0\" target=\"\\166\\057\\061\\001\" />"
     "  <magic name=\"FAX\" offset=\"0\" target=\"DFAX\" />"
     "  <magic name=\"FIG\" offset=\"0\" target=\"#FIG\" />"
     "  <magic name=\"FITS\" offset=\"0\" target=\"IT0\" />"
@@ -126,6 +130,8 @@ static const char
     "  <magic name=\"PNM\" offset=\"0\" target=\"P5\" />"
     "  <magic name=\"PNM\" offset=\"0\" target=\"P6\" />"
     "  <magic name=\"PNM\" offset=\"0\" target=\"P7\" />"
+    "  <magic name=\"PNM\" offset=\"0\" target=\"PF\" />"
+    "  <magic name=\"PNM\" offset=\"0\" target=\"Pf\" />"
     "  <magic name=\"PS\" offset=\"0\" target=\"%!\" />"
     "  <magic name=\"PS\" offset=\"0\" target=\"\\004%!\" />"
     "  <magic name=\"PS\" offset=\"0\" target=\"\\305\\320\\323\\306\" />"
@@ -143,6 +149,8 @@ static const char
     "  <magic name=\"TXT\" offset=\"0\" target=\"# ImageMagick pixel enumeration:\" />"
     "  <magic name=\"TIFF\" offset=\"0\" target=\"\\115\\115\\000\\052\" />"
     "  <magic name=\"TIFF\" offset=\"0\" target=\"\\111\\111\\052\\000\" />"
+    "  <magic name=\"TIFF64\" offset=\"0\" target=\"\\115\\115\\000\\053\\000\\010\\000\\000\" />"
+    "  <magic name=\"TIFF64\" offset=\"0\" target=\"\\115\\115\\000\\053\\000\\010\\000\\000\" />"
     "  <magic name=\"VICAR\" offset=\"0\" target=\"LBLSIZE\" />"
     "  <magic name=\"VICAR\" offset=\"0\" target=\"NJPL1I\" />"
     "  <magic name=\"VIFF\" offset=\"0\" target=\"\\253\\001\" />"
@@ -189,7 +197,6 @@ static MagickBooleanType
 %
 %      DestroyMagicList(void)
 %
-%
 */
 
 static void *DestroyMagicElement(void *magic_info)
@@ -199,11 +206,11 @@ static void *DestroyMagicElement(void *magic_info)
 
   p=(MagicInfo *) magic_info;
   if (p->path != (char *) NULL)
-    p->path=(char *) RelinquishMagickMemory(p->path);
+    p->path=DestroyString(p->path);
   if (p->name != (char *) NULL)
-    p->name=(char *) RelinquishMagickMemory(p->name);
+    p->name=DestroyString(p->name);
   if (p->target != (char *) NULL)
-    p->target=(char *) RelinquishMagickMemory(p->target);
+    p->target=DestroyString(p->target);
   if (p->magic != (unsigned char *) NULL)
     p->magic=(unsigned char *) RelinquishMagickMemory(p->magic);
   p=(MagicInfo *) RelinquishMagickMemory(p);
@@ -217,7 +224,7 @@ MagickExport void DestroyMagicList(void)
     magic_list=DestroyLinkedList(magic_list,DestroyMagicElement);
   instantiate_magic=MagickFalse;
   RelinquishSemaphoreInfo(magic_semaphore);
-  magic_semaphore=DestroySemaphoreInfo(magic_semaphore);
+  DestroySemaphoreInfo(&magic_semaphore);
 }
 
 /*
@@ -244,10 +251,9 @@ MagickExport void DestroyMagicList(void)
 %    o magic: A binary string generally representing the first few characters
 %      of the image file or blob.
 %
-%    o length: The length of the binary signature.
-%
-%    o exception: Return any errors or warnings in this structure.
+%    o length: the length of the binary signature.
 %
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
@@ -269,7 +275,7 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
   if (length == 0)
     return((const MagicInfo *) NULL);
   /*
-    Search for requested magic.
+    Search for magic tag.
   */
   AcquireSemaphoreInfo(&magic_semaphore);
   ResetLinkedListIterator(magic_list);
@@ -282,8 +288,11 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
       break;
     p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   }
+  if (p != (const MagicInfo *) NULL)
+    (void) InsertValueInLinkedList(magic_list,0,
+      RemoveElementByValueFromLinkedList(magic_list,p));
   RelinquishSemaphoreInfo(magic_semaphore);
-  return((const MagicInfo *) p);
+  return(p);
 }
 
 /*
@@ -309,10 +318,10 @@ MagickExport const MagicInfo *GetMagicInfo(const unsigned char *magic,
 %
 %    o pattern: Specifies a pointer to a text string containing a pattern.
 %
-%    o number_aliases:  This integer returns the number of magics in the
+%    o number_aliases:  This integer returns the number of aliases in the
 %      list.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -359,8 +368,8 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
   p=GetMagicInfo((const unsigned char *) "*",0,exception);
   if (p == (const MagicInfo *) NULL)
     return((const MagicInfo **) NULL);
-  aliases=(const MagicInfo **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(magic_list)+1)*sizeof(*aliases));
+  aliases=(const MagicInfo **) AcquireQuantumMemory((size_t)
+    GetNumberOfElementsInLinkedList(magic_list)+1UL,sizeof(*aliases));
   if (aliases == (const MagicInfo **) NULL)
     return((const MagicInfo **) NULL);
   /*
@@ -372,7 +381,7 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
   for (i=0; p != (const MagicInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
-        (GlobExpression(p->name,pattern) != MagickFalse))
+        (GlobExpression(p->name,pattern,MagickFalse) != MagickFalse))
       aliases[i++]=p;
     p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   }
@@ -409,7 +418,7 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
 %    o number_aliases:  This integer returns the number of image format aliases
 %      in the list.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -454,8 +463,8 @@ MagickExport char **GetMagicList(const char *pattern,
   p=GetMagicInfo((const unsigned char *) "*",0,exception);
   if (p == (const MagicInfo *) NULL)
     return((char **) NULL);
-  aliases=(char **) AcquireMagickMemory((size_t)
-    (GetNumberOfElementsInLinkedList(magic_list)+1)*sizeof(*aliases));
+  aliases=(char **) AcquireQuantumMemory((size_t)
+    GetNumberOfElementsInLinkedList(magic_list)+1UL,sizeof(*aliases));
   if (aliases == (char **) NULL)
     return((char **) NULL);
   AcquireSemaphoreInfo(&magic_semaphore);
@@ -464,8 +473,8 @@ MagickExport char **GetMagicList(const char *pattern,
   for (i=0; p != (const MagicInfo *) NULL; )
   {
     if ((p->stealth == MagickFalse) &&
-        (GlobExpression(p->name,pattern) != MagickFalse))
-      aliases[i++]=ConstantString(AcquireString(p->name));
+        (GlobExpression(p->name,pattern,MagickFalse) != MagickFalse))
+      aliases[i++]=ConstantString(p->name);
     p=(const MagicInfo *) GetNextValueInLinkedList(magic_list);
   }
   RelinquishSemaphoreInfo(magic_semaphore);
@@ -524,7 +533,7 @@ MagickExport const char *GetMagicName(const MagicInfo *magic_info)
 %
 %  A description of each parameter follows.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 static MagickBooleanType InitializeMagicList(ExceptionInfo *exception)
@@ -565,8 +574,7 @@ static MagickBooleanType InitializeMagicList(ExceptionInfo *exception)
 %
 %    o file:  An pointer to a FILE.
 %
-%    o exception: Return any errors or warnings in this structure.
-%
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport MagickBooleanType ListMagicInfo(FILE *file,
@@ -648,8 +656,7 @@ MagickExport MagickBooleanType ListMagicInfo(FILE *file,
 %
 %    o depth: depth of <include /> statements.
 %
-%    o exception: Return any errors or warnings in this structure.
-%
+%    o exception: return any errors or warnings in this structure.
 %
 */
 static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
@@ -657,33 +664,28 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
 {
   char
     keyword[MaxTextExtent],
-    *q,
     *token;
 
-  MagicInfo
-    *magic_info = (MagicInfo *) NULL;
+  const char
+    *q;
 
-  MagickStatusType
+  MagickBooleanType
     status;
 
+  MagicInfo
+    *magic_info;
+
   /*
     Load the magic map file.
   */
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-    "Loading magic file \"%s\" ...",filename);
+    "Loading magic configure file \"%s\" ...",filename);
   if (xml == (char *) NULL)
     return(MagickFalse);
   if (magic_list == (LinkedListInfo *) NULL)
-    {
-      magic_list=NewLinkedList(0);
-      if (magic_list == (LinkedListInfo *) NULL)
-        {
-          ThrowFileException(exception,ResourceLimitError,
-            "MemoryAllocationFailed",filename);
-          return(MagickFalse);
-        }
-    }
+    magic_list=NewLinkedList(0);
   status=MagickTrue;
+  magic_info=(MagicInfo *) NULL;
   token=AcquireString(xml);
   for (q=(char *) xml; *q != '\0'; )
   {
@@ -739,10 +741,16 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
                   if (*path != '\0')
                     (void) ConcatenateMagickString(path,DirectorySeparator,
                       MaxTextExtent);
-                  (void) ConcatenateMagickString(path,token,MaxTextExtent);
+                  if (*token == *DirectorySeparator)
+                    (void) CopyMagickString(path,token,MaxTextExtent);
+                  else
+                    (void) ConcatenateMagickString(path,token,MaxTextExtent);
                   xml=FileToString(path,~0,exception);
-                  status|=LoadMagicList(xml,path,depth+1,exception);
-                  xml=(char *) RelinquishMagickMemory(xml);
+                  if (xml != (char *) NULL)
+                    {
+                      status=LoadMagicList(xml,path,depth+1,exception);
+                      xml=(char *) RelinquishMagickMemory(xml);
+                    }
                 }
             }
         }
@@ -750,15 +758,14 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
       }
     if (LocaleCompare(keyword,"<magic") == 0)
       {
-       /*
-          Allocate memory for the magic list.
+        /*
+          Magic element.
         */
         magic_info=(MagicInfo *) AcquireMagickMemory(sizeof(*magic_info));
         if (magic_info == (MagicInfo *) NULL)
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",filename);
+          ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
         (void) ResetMagickMemory(magic_info,0,sizeof(*magic_info));
-        magic_info->path=ConstantString(AcquireString(filename));
+        magic_info->path=ConstantString(filename);
         magic_info->signature=MagickSignature;
         continue;
       }
@@ -773,7 +780,7 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
             magic_info->name);
         magic_info=(MagicInfo *) NULL;
       }
-    GetMagickToken(q,(char **) NULL,token);
+    GetMagickToken(q,(const char **) NULL,token);
     if (*token != '=')
       continue;
     GetMagickToken(q,&q,token);
@@ -785,7 +792,7 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"name") == 0)
           {
-            magic_info->name=ConstantString(AcquireString(token));
+            magic_info->name=ConstantString(token);
             break;
           }
         break;
@@ -805,8 +812,7 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
       {
         if (LocaleCompare((char *) keyword,"stealth") == 0)
           {
-            magic_info->stealth=(MagickBooleanType)
-              (LocaleCompare(token,"True") == 0);
+            magic_info->stealth=IsMagickTrue(token);
             break;
           }
         break;
@@ -822,8 +828,8 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
             register unsigned char
               *q;
 
-            magic_info->target=AcquireString(token);
-            magic_info->magic=(unsigned char *) AcquireString(token);
+            magic_info->target=ConstantString(token);
+            magic_info->magic=(unsigned char *) ConstantString(token);
             q=magic_info->magic;
             for (p=magic_info->target; *p != '\0'; )
             {
@@ -860,6 +866,7 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
               *q++=(unsigned char) (*p++);
               magic_info->length++;
             }
+            (void) SubstituteString((char **) &magic_info->target,"&amp;","&");
             break;
           }
         break;
@@ -869,9 +876,7 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
     }
   }
   token=(char *) RelinquishMagickMemory(token);
-  if (magic_list == (LinkedListInfo *) NULL)
-    return(MagickFalse);
-  return(status != 0 ? MagickTrue : MagickFalse);
+  return(status);
 }
 
 /*
@@ -895,16 +900,15 @@ static MagickBooleanType LoadMagicList(const char *xml,const char *filename,
 %
 %  A description of each parameter follows:
 %
-%    o filename: The font file name.
-%
-%    o exception: Return any errors or warnings in this structure.
+%    o filename: the font file name.
 %
+%    o exception: return any errors or warnings in this structure.
 %
 */
 static MagickBooleanType LoadMagicLists(const char *filename,
   ExceptionInfo *exception)
 {
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
   return(LoadMagicList(MagicMap,"built-in",0,exception));
 #else
   char
@@ -925,9 +929,9 @@ static MagickBooleanType LoadMagicLists(const char *filename,
   option=(const StringInfo *) GetNextValueInLinkedList(options);
   while (option != (const StringInfo *) NULL)
   {
-    (void) CopyMagickString(path,option->path,MaxTextExtent);
-    status|=LoadMagicList((const char *) option->datum,option->path,0,
-      exception);
+    (void) CopyMagickString(path,GetStringInfoPath(option),MaxTextExtent);
+    status|=LoadMagicList((const char *) GetStringInfoDatum(option),
+      GetStringInfoPath(option),0,exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
   options=DestroyConfigureOptions(options);
