@@ -10,14 +10,14 @@
 %                    M   M  EEEEE  M   M   OOO   R  R     Y                   %
 %                                                                             %
 %                                                                             %
-%                    ImageMagick Memory Allocation Methods                    %
+%                     MagickCore Memory Allocation Methods                    %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,11 +43,10 @@
 %  sizes.  It is based on the paper, "Fast Memory Allocation using Lazy Fits."
 %  written by Yoo C. Chung.
 %
-%  By default, ANSI memory methods are called (e.g. malloc()).  Use the
-%  custom memory allocator by defining UseEmbeddableMagick.  The custom memory
-%  allocator has the advantage of using memory-mapped heap allocation which
-%  permits the virtual memory to be returned to the system in all cases, unlike
-%  the ANSI memory methods that typically only memory-map large allocations.
+%  By default, ANSI memory methods are called (e.g. malloc).  Use the
+%  custom memory allocator by defining MAGICKCORE_EMBEDDABLE_SUPPORT
+%  to allocate memory with private anonymous mapping rather than from the
+%  heap.
 %
 */
 
@@ -119,11 +118,26 @@ typedef struct _MemoryInfo
     *segments[MaxSegments],
     segment_pool[MaxSegments];
 } MemoryInfo;
+
+typedef struct _MagickMemoryMethods
+{
+  AcquireMemoryHandler
+    acquire_memory_handler;
+
+  ResizeMemoryHandler
+    resize_memory_handler;
+
+  DestroyMemoryHandler
+    destroy_memory_handler;
+} MagickMemoryMethods;
 
 /*
   Global declarations.
 */
-#if defined(UseEmbeddableMagick)
+static MagickMemoryMethods
+  memory_methods = { malloc, realloc, free };
+
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
 static MemoryInfo
   memory_info;
 
@@ -140,7 +154,7 @@ static MagickBooleanType
   ExpandHeap(size_t);
 #endif
 
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -161,7 +175,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o size: The size of the memory in bytes to allocate.
+%    o size: the size of the memory in bytes to allocate.
 %
 */
 
@@ -247,7 +261,7 @@ static void *AcquireBlock(size_t size)
   /*
     Find free block.
   */
-  size=(size_t) (size+sizeof(size_t)+6*sizeof(size_t)-1) & -(4*sizeof(size_t));
+  size=(size_t) (size+sizeof(size_t)+6*sizeof(size_t)-1) & -(4U*sizeof(size_t));
   i=AllocationPolicy(size);
   block=memory_info.blocks[i];
   while ((block != (void *) NULL) && (SizeOfBlock(block) < size))
@@ -309,7 +323,7 @@ static void *AcquireBlock(size_t size)
 %
 %  A description of each parameter follows:
 %
-%    o size: The size of the memory in bytes to allocate.
+%    o size: the size of the memory in bytes to allocate.
 %
 */
 MagickExport void *AcquireMagickMemory(const size_t size)
@@ -317,8 +331,8 @@ MagickExport void *AcquireMagickMemory(const size_t size)
   register void
     *memory;
 
-#if !defined(UseEmbeddableMagick)
-  memory=malloc(size == 0 ? 1UL : size);
+#if !defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
+  memory=memory_methods.acquire_memory_handler(size == 0 ? 1UL : size);
 #else
   if (free_segments == (DataSegmentInfo *) NULL)
     {
@@ -361,6 +375,45 @@ MagickExport void *AcquireMagickMemory(const size_t size)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   A c q u i r e Q u a n t u m M e m o r y                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AcquireQuantumMemory() returns a pointer to a block of memory at least
+%  count * quantum bytes suitably aligned for any use.
+%
+%  The format of the AcquireQuantumMemory method is:
+%
+%      void *AcquireQuantumMemory(const size_t count,const size_t quantum)
+%
+%  A description of each parameter follows:
+%
+%    o count: the number of quantum elements to allocate.
+%
+%    o quantum: the number of bytes in each quantum.
+%
+*/
+MagickExport void *AcquireQuantumMemory(const size_t count,const size_t quantum)
+{
+  size_t
+    size;
+
+  size=count*quantum;
+  if ((count == 0) || (quantum != (size/count)))
+    {
+      errno=ENOMEM;
+      return((void *) NULL);
+    }
+  return(AcquireMagickMemory(size));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   C o p y M a g i c k M e m o r y                                           %
 %                                                                             %
 %                                                                             %
@@ -378,12 +431,11 @@ MagickExport void *AcquireMagickMemory(const size_t size)
 %
 %  A description of each parameter follows:
 %
-%    o destination: The destination.
+%    o destination: the destination.
 %
-%    o source: The source.
-%
-%    o size: The size of the memory in bytes to allocate.
+%    o source: the source.
 %
+%    o size: the size of the memory in bytes to allocate.
 %
 */
 MagickExport void *CopyMagickMemory(void *destination,const void *source,
@@ -397,12 +449,21 @@ MagickExport void *CopyMagickMemory(void *destination,const void *source,
 
   assert(destination != (void *) NULL);
   assert(source != (const void *) NULL);
-  if ((size == 0) || (source == destination))
-    return(destination);
   p=(const unsigned char *) source;
   q=(unsigned char *) destination;
-  if ((q >= (p+size)) || (p >= (q+size)))
-    return(memcpy(destination,source,size));
+  if (((q+size) < p) || (q > (p+size)))
+    switch (size)
+    {
+      default: return(memcpy(destination,source,size));
+      case 7: *q++=(*p++);
+      case 6: *q++=(*p++);
+      case 5: *q++=(*p++);
+      case 4: *q++=(*p++);
+      case 3: *q++=(*p++);
+      case 2: *q++=(*p++);
+      case 1: *q++=(*p++);
+      case 0: return(destination);
+    }
   return(memmove(destination,source,size));
 }
 
@@ -426,7 +487,7 @@ MagickExport void *CopyMagickMemory(void *destination,const void *source,
 */
 MagickExport void DestroyMagickMemory(void)
 {
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
   register long
     i;
 
@@ -434,17 +495,18 @@ MagickExport void DestroyMagickMemory(void)
   RelinquishSemaphoreInfo(memory_semaphore);
   for (i=0; i < (long) memory_info.number_segments; i++)
     if (memory_info.segments[i]->mapped == MagickFalse)
-      free(memory_info.segments[i]->allocation);
+      memory_methods.destroy_memory_handler(
+        memory_info.segments[i]->allocation);
     else
       (void) UnmapBlob(memory_info.segments[i]->allocation,
         memory_info.segments[i]->length);
   free_segments=(DataSegmentInfo *) NULL;
   (void) ResetMagickMemory(&memory_info,0,sizeof(memory_info));
-  memory_semaphore=DestroySemaphoreInfo(memory_semaphore);
+  DestroySemaphoreInfo(&memory_semaphore);
 #endif
 }
 
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -465,7 +527,7 @@ MagickExport void DestroyMagickMemory(void)
 %
 %  A description of each parameter follows:
 %
-%    o size: The size of the memory in bytes we require.
+%    o size: the size of the memory in bytes we require.
 %
 */
 static MagickBooleanType ExpandHeap(size_t size)
@@ -493,7 +555,7 @@ static MagickBooleanType ExpandHeap(size_t size)
   segment=MapBlob(-1,IOMode,0,blocksize);
   mapped=segment != (void *) NULL ? MagickTrue : MagickFalse;
   if (segment == (void *) NULL)
-    segment=(void *) malloc(blocksize);
+    segment=(void *) memory_methods.acquire_memory_handler(blocksize);
   if (segment == (void *) NULL)
     return(MagickFalse);
   segment_info=(DataSegmentInfo *) free_segments;
@@ -525,6 +587,48 @@ static MagickBooleanType ExpandHeap(size_t size)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t M a g i c k M e m o r y M e t h o d s                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetMagickMemoryMethods() gets the methods to acquire, resize, and destroy
+%  memory.
+%
+%  The format of the GetMagickMemoryMethods() method is:
+%
+%      void GetMagickMemoryMethods(AcquireMemoryHandler *acquire_memory_handler,
+%        ResizeMemoryHandler *resize_memory_handler,
+%        DestroyMemoryHandler *destroy_memory_handler)
+%
+%  A description of each parameter follows:
+%
+%    o acquire_memory_handler: method to acquire memory (e.g. malloc).
+%
+%    o resize_memory_handler: method to resize memory (e.g. realloc).
+%
+%    o destroy_memory_handler: method to destroy memory (e.g. free).
+%
+*/
+MagickExport void GetMagickMemoryMethods(
+  AcquireMemoryHandler *acquire_memory_handler,
+  ResizeMemoryHandler *resize_memory_handler,
+  DestroyMemoryHandler *destroy_memory_handler)
+{
+  assert(acquire_memory_handler != (AcquireMemoryHandler *) NULL);
+  assert(resize_memory_handler != (ResizeMemoryHandler *) NULL);
+  assert(destroy_memory_handler != (DestroyMemoryHandler *) NULL);
+  *acquire_memory_handler=memory_methods.acquire_memory_handler;
+  *resize_memory_handler=memory_methods.resize_memory_handler;
+  *destroy_memory_handler=memory_methods.destroy_memory_handler;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   R e l i n q u i s h M a g i c k M e m o r y                               %
 %                                                                             %
 %                                                                             %
@@ -547,8 +651,8 @@ MagickExport void *RelinquishMagickMemory(void *memory)
 {
   if (memory == (void *) NULL)
     return((void *) NULL);
-#if !defined(UseEmbeddableMagick)
-  free(memory);
+#if !defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
+  memory_methods.destroy_memory_handler(memory);
 #else
   assert((SizeOfBlock(memory) % (4*sizeof(size_t))) == 0);
   assert((*BlockHeader(NextBlock(memory)) & PreviousBlockBit) != 0);
@@ -644,11 +748,11 @@ MagickExport void *ResetMagickMemory(void *memory,int byte,const size_t size)
 %
 %    o memory: A pointer to a memory allocation.
 %
-%    o size: The new size of the allocated memory.
+%    o size: the new size of the allocated memory.
 %
 */
 
-#if defined(UseEmbeddableMagick)
+#if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
 static inline void *ResizeBlock(void *block,size_t size)
 {
   register void
@@ -675,8 +779,10 @@ MagickExport void *ResizeMagickMemory(void *memory,const size_t size)
 
   if (memory == (void *) NULL)
     return(AcquireMagickMemory(size));
-#if !defined(UseEmbeddableMagick)
-  block=realloc(memory,size == 0 ? 1UL : size);
+#if !defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
+  block=memory_methods.resize_memory_handler(memory,size == 0 ? 1UL : size);
+  if (block == (void *) NULL)
+    memory=RelinquishMagickMemory(memory);
 #else
   AcquireSemaphoreInfo(&memory_semaphore);
   block=ResizeBlock(memory,size == 0 ? 1UL : size);
@@ -686,15 +792,103 @@ MagickExport void *ResizeMagickMemory(void *memory,const size_t size)
         {
           RelinquishSemaphoreInfo(memory_semaphore);
           memory=RelinquishMagickMemory(memory);
-          ThrowMagickFatalException(ResourceLimitFatalError,
-            "MemoryAllocationFailed",strerror(errno));
+          ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
         }
       block=ResizeBlock(memory,size == 0 ? 1UL : size);
       assert(block != (void *) NULL);
     }
   RelinquishSemaphoreInfo(memory_semaphore);
+  memory=RelinquishMagickMemory(memory);
 #endif
-  if (block == (void *) NULL)
-    memory=RelinquishMagickMemory(memory);
   return(block);
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   R e s i z e Q u a n t u m M e m o r y                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ResizeQuantumMemory() changes the size of the memory and returns a pointer
+%  to the (possibly moved) block.  The contents will be unchanged up to the
+%  lesser of the new and old sizes.
+%
+%  The format of the ResizeQuantumMemory method is:
+%
+%      void *ResizeQuantumMemory(void *memory,const size_t count,
+%        const size_t quantum)
+%
+%  A description of each parameter follows:
+%
+%    o memory: A pointer to a memory allocation.
+%
+%    o count: the number of quantum elements to allocate.
+%
+%    o quantum: the number of bytes in each quantum.
+%
+*/
+MagickExport void *ResizeQuantumMemory(void *memory,const size_t count,
+  const size_t quantum)
+{
+  size_t
+    size;
+
+  size=count*quantum;
+  if ((count == 0) || (quantum != (size/count)))
+    {
+      memory=RelinquishMagickMemory(memory);
+      errno=ENOMEM;
+      return((void *) NULL);
+    }
+  return(ResizeMagickMemory(memory,size));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   S e t M a g i c k M e m o r y M e t h o d s                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetMagickMemoryMethods() sets the methods to acquire, resize, and destroy
+%  memory.
+%
+%  The format of the SetMagickMemoryMethods() method is:
+%
+%      SetMagickMemoryMethods(AcquireMemoryHandler acquire_memory_handler,
+%        ResizeMemoryHandler resize_memory_handler,
+%        DestroyMemoryHandler destroy_memory_handler)
+%
+%  A description of each parameter follows:
+%
+%    o acquire_memory_handler: method to acquire memory (e.g. malloc).
+%
+%    o resize_memory_handler: method to resize memory (e.g. realloc).
+%
+%    o destroy_memory_handler: method to destroy memory (e.g. free).
+%
+*/
+MagickExport void SetMagickMemoryMethods(
+  AcquireMemoryHandler acquire_memory_handler,
+  ResizeMemoryHandler resize_memory_handler,
+  DestroyMemoryHandler destroy_memory_handler)
+{
+  /*
+    Set memory methods.
+  */
+  if (acquire_memory_handler != (AcquireMemoryHandler) NULL)
+    memory_methods.acquire_memory_handler=acquire_memory_handler;
+  if (resize_memory_handler != (ResizeMemoryHandler) NULL)
+    memory_methods.resize_memory_handler=resize_memory_handler;
+  if (destroy_memory_handler != (DestroyMemoryHandler) NULL)
+    memory_methods.destroy_memory_handler=destroy_memory_handler;
+}
