@@ -10,14 +10,14 @@
 %                      T     OOO   K   K  EEEEE  N   N                        %
 %                                                                             %
 %                                                                             %
-%                        ImageMagick Token Methods                            %
+%                         MagickCore Token Methods                            %
 %                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -49,6 +49,90 @@
 #include "magick/token.h"
 #include "magick/utility.h"
 
+/*
+  Typedef declaractions.
+*/
+struct _TokenInfo
+{
+  int
+    state;
+
+  MagickStatusType
+    flag;
+
+  long
+    offset;
+
+  char
+    quote;
+
+  unsigned long
+    signature;
+};
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   A c q u i r e T o k e n I n f o                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AcquireTokenInfo() allocates the TokenInfo structure.
+%
+%  The format of the AcquireTokenInfo method is:
+%
+%      TokenInfo *AcquireTokenInfo()
+%
+*/
+MagickExport TokenInfo *AcquireTokenInfo(void)
+{
+  TokenInfo
+    *token_info;
+
+  token_info=(TokenInfo *) AcquireMagickMemory(sizeof(*token_info));
+  if (token_info == (TokenInfo *) NULL)
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  token_info->signature=MagickSignature;
+  return(token_info);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   D e s t r o y T o k e n I n f o                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  DestroyTokenInfo() deallocates memory associated with an TokenInfo
+%  structure.
+%
+%  The format of the DestroyTokenInfo method is:
+%
+%      TokenInfo *DestroyTokenInfo(TokenInfo *token_info)
+%
+%  A description of each parameter follows:
+%
+%    o token_info: Specifies a pointer to an TokenInfo structure.
+%
+*/
+MagickExport TokenInfo *DestroyTokenInfo(TokenInfo *token_info)
+{
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  assert(token_info != (TokenInfo *) NULL);
+  assert(token_info->signature == MagickSignature);
+  token_info->signature=(~MagickSignature);
+  token_info=(TokenInfo *) RelinquishMagickMemory(token_info);
+  return(token_info);
+}
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -67,7 +151,7 @@
 %
 %  The format of the GetMagickToken method is:
 %
-%      void GetMagickToken(const char *start,char **end,char *token)
+%      void GetMagickToken(const char *start,const char **end,char *token)
 %
 %  A description of each parameter follows:
 %
@@ -77,10 +161,12 @@
 %
 %    o token: copy the token to this buffer.
 %
-%
 */
-MagickExport void GetMagickToken(const char *start,char **end,char *token)
+MagickExport void GetMagickToken(const char *start,const char **end,char *token)
 {
+  double
+    value;
+
   register const char
     *p;
 
@@ -138,10 +224,10 @@ MagickExport void GetMagickToken(const char *start,char **end,char *token)
         char
           *q;
 
-        (void) strtod(p,&q);
-        if (p != q)
+        value=strtod(p,&q);
+        if ((p != q) && (*p != ','))
           {
-            for ( ; p < q; p++)
+            for ( ; (p < q) && (*p != ','); p++)
               token[i++]=(*p);
             if (*p == '%')
               token[i++]=(*p++);
@@ -156,11 +242,13 @@ MagickExport void GetMagickToken(const char *start,char **end,char *token)
         for ( ; *p != '\0'; p++)
         {
           if (((isspace((int) ((unsigned char) *p)) != 0) || (*p == '=') ||
-               (*p == ',') || (*p == ':')) && (*(p-1) != '\\'))
+              (*p == ',') || (*p == ':')) && (*(p-1) != '\\'))
             break;
           if ((i > 0) && (*p == '<'))
             break;
           token[i++]=(*p);
+          if (*p == '>')
+            break;
           if (*p == '(')
             for (p++; *p != '\0'; p++)
             {
@@ -175,22 +263,22 @@ MagickExport void GetMagickToken(const char *start,char **end,char *token)
     break;
   }
   token[i]='\0';
-  if (LocaleNCompare(token,"url(#",5) == 0)
-    {
-      i=(long) strlen(token);
-      (void) CopyMagickString(token,token+5,MaxTextExtent);
-      token[i-6]='\0';
-    }
   if (LocaleNCompare(token,"url(",4) == 0)
     {
+      ssize_t
+        offset;
+
+      offset=4;
+      if (token[offset] == '#')
+        offset++;
       i=(long) strlen(token);
-      (void) CopyMagickString(token,token+4,MaxTextExtent);
-      token[i-5]='\0';
+      (void) CopyMagickString(token,token+offset,MaxTextExtent);
+      token[i-offset-1]='\0';
     }
   while (isspace((int) ((unsigned char) *p)) != 0)
     p++;
-  if (end != (char **) NULL)
-    *end=(char *) p;
+  if (end != (const char **) NULL)
+    *end=(const char *) p;
 }
 
 /*
@@ -209,7 +297,7 @@ MagickExport void GetMagickToken(const char *start,char **end,char *token)
 %  The format of the GlobExpression function is:
 %
 %      MagickBooleanType GlobExpression(const char *expression,
-%        const char *pattern)
+%        const char *pattern,const MagickBooleanType case_insensitive)
 %
 %  A description of each parameter follows:
 %
@@ -217,13 +305,16 @@ MagickExport void GetMagickToken(const char *start,char **end,char *token)
 %
 %    o pattern: Specifies a pointer to a text string containing a pattern.
 %
+%    o case_insensitive: set to MagickTrue to ignore the case when matching
+%      an expression.
 %
 */
 MagickExport MagickBooleanType GlobExpression(const char *expression,
-  const char *pattern)
+  const char *pattern,const MagickBooleanType case_insensitive)
 {
   MagickBooleanType
-    done;
+    done,
+    match;
 
   register const char
     *p;
@@ -241,7 +332,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
   if ((*p == ']') && (strchr(pattern,'[') != (char *) NULL))
     {
       ExceptionInfo
-        exception;
+        *exception;
 
       ImageInfo
         *image_info;
@@ -249,11 +340,11 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
       /*
         Determine if pattern is a scene, i.e. img0001.pcd[2].
       */
-      image_info=CloneImageInfo((ImageInfo *) NULL);
+      image_info=AcquireImageInfo();
       (void) CopyMagickString(image_info->filename,pattern,MaxTextExtent);
-      GetExceptionInfo(&exception);
-      (void) SetImageInfo(image_info,MagickTrue,&exception);
-      DestroyExceptionInfo(&exception);
+      exception=AcquireExceptionInfo();
+      (void) SetImageInfo(image_info,MagickTrue,exception);
+      exception=DestroyExceptionInfo(exception);
       if (LocaleCompare(image_info->filename,pattern) != 0)
         {
           image_info=DestroyImageInfo(image_info);
@@ -287,7 +378,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
         pattern++;
         status=MagickFalse;
         while ((*expression != '\0') && (status == MagickFalse))
-          status=GlobExpression(expression++,pattern);
+          status=GlobExpression(expression++,pattern,case_insensitive);
         if (status != MagickFalse)
           {
             while (*expression != '\0')
@@ -373,9 +464,6 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
       }
       case '{':
       {
-        MagickBooleanType
-          match;
-
         register const char
           *p;
 
@@ -390,7 +478,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
           {
             if (*pattern == '\\')
               pattern++;
-            match=(MagickBooleanType) (*pattern == *p);
+            match=(*pattern == *p) ? MagickTrue : MagickFalse;
             p++;
             pattern++;
           }
@@ -436,25 +524,37 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
       }
       default:
       {
-        if (*expression != *pattern)
-          done=MagickTrue;
-        else
+        if (case_insensitive != MagickFalse)
           {
-            expression++;
-            pattern++;
+            if (tolower((int) ((unsigned char) *expression)) !=
+                tolower((int) ((unsigned char) *pattern)))
+              {
+                done=MagickTrue;
+                break;
+              }
           }
+        else
+          if (*expression != *pattern)
+            {
+              done=MagickTrue;
+              break;
+            }
+        expression++;
+        pattern++;
       }
     }
   }
-  while (*pattern == '*')
+  while (*pattern == '*') 
     pattern++;
-  return((MagickBooleanType) ((*expression == '\0') && (*pattern == '\0')));
+  match=(*expression == '\0') && (*pattern == '\0') ? MagickTrue : MagickFalse;
+  return(match);
 }
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
 +     I s G l o b                                                             %
 %                                                                             %
 %                                                                             %
@@ -462,7 +562,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  IsGlob() returns MagickTrue if the path specification contains a globbing
-%  patten.
+%  pattern.
 %
 %  The format of the IsGlob method is:
 %
@@ -470,8 +570,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
 %
 %  A description of each parameter follows:
 %
-%    o path: The path.
-%
+%    o path: the path.
 %
 */
 MagickExport MagickBooleanType IsGlob(const char *path)
@@ -479,12 +578,14 @@ MagickExport MagickBooleanType IsGlob(const char *path)
   MagickBooleanType
     status;
 
-  status=(MagickBooleanType) ((strchr(path,'*') != (char *) NULL) ||
+  if (IsPathAccessible(path) != MagickFalse)
+    return(MagickFalse);
+  status=(strchr(path,'*') != (char *) NULL) ||
     (strchr(path,'?') != (char *) NULL) ||
     (strchr(path,'{') != (char *) NULL) ||
     (strchr(path,'}') != (char *) NULL) ||
     (strchr(path,'[') != (char *) NULL) ||
-    (strchr(path,']') != (char *) NULL));
+    (strchr(path,']') != (char *) NULL) ? MagickTrue : MagickFalse;
   return(status);
 }
 
@@ -698,7 +799,7 @@ static void StoreToken(TokenInfo *token_info,char *string,
 
 MagickExport int Tokenizer(TokenInfo *token_info,const unsigned flag,
   char *token,const size_t max_token_length,const char *line,const char *white,
-	const char *break_set,const char *quote,const char escape,char *breaker,
+  const char *break_set,const char *quote,const char escape,char *breaker,
   int *next,char *quoted)
 {
   int
