@@ -1,5 +1,5 @@
 #
-# Package name and versioning information for Magick
+# Package name and versioning information for ImageMagick.
 #
 # This file is sourced by a Bourne shell (/bin/sh) script so it must
 # observe Bourne shell syntax.
@@ -10,32 +10,26 @@ PACKAGE_NAME='ImageMagick'
 #
 # Package version.  This is is the numeric version suffix applied to
 # PACKAGE_NAME (e.g. "1.0.0").
-PACKAGE_VERSION='6.2.4'
-PACKAGE_LIB_VERSION="0x624"
-PACKAGE_LIB_VERSION_NUMBER="6,2,4,5"
-PACKAGE_RELEASE_DATE=`date +%x`
+PACKAGE_VERSION='6.5.1'
+PACKAGE_LIB_VERSION="0x651"
+PACKAGE_RELEASE="0"
+PACKAGE_LIB_VERSION_NUMBER="6,5,1,${PACKAGE_RELEASE}"
+PACKAGE_RELEASE_DATE=`date +%F`
 
 #
-# CVS branch that this release is on.
+# Date of last ChangeLog update
 #
-CVS_BRANCH_TAG=HEAD
+PACKAGE_CHANGE_DATE=`awk '/^[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]/ { print substr($1,1,4) substr($1,6,2) substr($1,9,2); exit; }' ${srcdir}/ChangeLog`
 
 #
-# Package version addendum.  This is an arbitrary suffix (if any)
-# appended to the package version. (e.g. "beta1")
-# `echo -snapshot-``date '+%g%m%d'`
-CHANGE_DATE=`find ${srcdir}/ChangeLog -printf '%Ty%Tm%Td%\n' 2> /dev/null`
-if test -n "$CHANGE_DATE"
-then
-  PACKAGE_VERSION_ADDENDUM=""
-else
-  PACKAGE_VERSION_ADDENDUM=""
-fi
+# Package version addendum.  This is an arbitrary suffix (if any) appended
+# to the package version. (e.g. "beta1")
+PACKAGE_LIB_SUBVERSION="-${PACKAGE_RELEASE}"
 
 #
-# Libtool library revision control info
-# See the libtool documentation under the heading "Libtool's versioning
-# system" in order to understand the meaning of these fields
+# Libtool library revision control info: See the libtool documentation under
+# the heading "Libtool's versioning system" in order to understand the meaning
+# of these fields.
 #
 # Here are a set of rules to help you update your library version
 # information:
@@ -52,6 +46,6 @@ fi
 #     increment age.
 #  6. If any interfaces have been removed since the last public release,
 #     then set age to 0.
-LIBRARY_CURRENT=9
-LIBRARY_REVISION=0
-LIBRARY_AGE=0
+MAGICK_LIBRARY_CURRENT=2
+MAGICK_LIBRARY_REVISION=0
+MAGICK_LIBRARY_AGE=0
